@@ -1,8 +1,7 @@
 #![allow(clippy::print_stdout)]
 
 use std::fs::File;
-use std::io::{self, stdout, BufWriter, Write};
-use std::num::NonZeroUsize;
+use std::io::{self, BufWriter, Write, stdout};
 use std::path::{Path, PathBuf};
 use std::process::ExitCode;
 use std::sync::mpsc::channel;
@@ -11,23 +10,24 @@ use anyhow::Result;
 use clap::CommandFactory;
 use colored::Colorize;
 use log::warn;
-use notify::{recommended_watcher, RecursiveMode, Watcher};
+use notify::{RecursiveMode, Watcher, recommended_watcher};
 
 use args::{GlobalConfigArgs, ServerCommand};
-use ruff_linter::logging::{set_up_logging, LogLevel};
+use ruff_linter::logging::{LogLevel, set_up_logging};
 use ruff_linter::settings::flags::FixMode;
 use ruff_linter::settings::types::OutputFormat;
 use ruff_linter::{fs, warn_user, warn_user_once};
 use ruff_workspace::Settings;
 
-use crate::args::{Args, CheckCommand, Command, FormatCommand};
+use crate::args::{
+    AnalyzeCommand, AnalyzeGraphCommand, Args, CheckCommand, Command, FormatCommand,
+};
 use crate::printer::{Flags as PrinterFlags, Printer};
 
 pub mod args;
 mod cache;
 mod commands;
 mod diagnostics;
-mod panic;
 mod printer;
 pub mod resolve;
 mod stdin;
@@ -62,11 +62,19 @@ enum ChangeKind {
 /// Return the [`ChangeKind`] based on the list of modified file paths.
 ///
 /// Returns `None` if no relevant changes were detected.
-fn change_detected(paths: &[PathBuf]) -> Option<ChangeKind> {
+fn change_detected(event: &notify::Event) -> Option<ChangeKind> {
     // If any `.toml` files were modified, return `ChangeKind::Configuration`. Otherwise, return
     // `ChangeKind::SourceFile` if any `.py`, `.pyi`, `.pyw`, or `.ipynb` files were modified.
     let mut source_file = false;
-    for path in paths {
+
+    if event.kind.is_access() || event.kind.is_other() {
+        return None;
+    }
+
+    if event.need_rescan() {
+        return Some(ChangeKind::Configuration);
+    }
+    for path in &event.paths {
         if let Some(suffix) = path.extension() {
             match suffix.to_str() {
                 Some("toml") => {
@@ -103,7 +111,7 @@ fn is_stdin(files: &[PathBuf], stdin_filename: Option<&Path>) -> bool {
     file == Path::new("-")
 }
 
-/// Returns the default set of files if none are provided, otherwise returns `None`.
+/// Returns the default set of files if none are provided, otherwise returns provided files.
 fn resolve_default_files(files: Vec<PathBuf>, is_stdin: bool) -> Vec<PathBuf> {
     if files.is_empty() {
         if is_stdin {
@@ -125,7 +133,7 @@ pub fn run(
     {
         let default_panic_hook = std::panic::take_hook();
         std::panic::set_hook(Box::new(move |info| {
-            #[allow(clippy::print_stderr)]
+            #[expect(clippy::print_stderr)]
             {
                 eprintln!(
                     r#"
@@ -144,7 +152,11 @@ pub fn run(
         }));
     }
 
-    set_up_logging(global_options.log_level())?;
+    // Don't set up logging for the server command, as it has its own logging setup
+    // and setting the global logger can only be done once.
+    if !matches!(command, Command::Server { .. }) {
+        set_up_logging(global_options.log_level())?;
+    }
 
     match command {
         Command::Version { output_format } => {
@@ -186,6 +198,7 @@ pub fn run(
         Command::Check(args) => check(args, global_options),
         Command::Format(args) => format(args, global_options),
         Command::Server(args) => server(args),
+        Command::Analyze(AnalyzeCommand::Graph(args)) => analyze_graph(args, global_options),
     }
 }
 
@@ -199,16 +212,17 @@ fn format(args: FormatCommand, global_options: GlobalConfigArgs) -> Result<ExitS
     }
 }
 
+fn analyze_graph(
+    args: AnalyzeGraphCommand,
+    global_options: GlobalConfigArgs,
+) -> Result<ExitStatus> {
+    let (cli, config_arguments) = args.partition(global_options)?;
+
+    commands::analyze_graph::analyze_graph(cli, &config_arguments)
+}
+
 fn server(args: ServerCommand) -> Result<ExitStatus> {
-    let ServerCommand { preview } = args;
-
-    let four = NonZeroUsize::new(4).unwrap();
-
-    // by default, we set the number of worker threads to `num_cpus`, with a maximum of 4.
-    let worker_threads = std::thread::available_parallelism()
-        .unwrap_or(four)
-        .max(four);
-    commands::server::run_server(preview, worker_threads)
+    commands::server::run_server(args.resolve_preview())
 }
 
 pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<ExitStatus> {
@@ -269,8 +283,8 @@ pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<Exi
     // - If `--fix` or `--fix-only` is set, apply applicable fixes to the filesystem (or
     //   print them to stdout, if we're reading from stdin).
     // - If `--diff` or `--fix-only` are set, don't print any violations (only applicable fixes)
-    // - By default, applicable fixes only include [`Applicablility::Automatic`], but if
-    //   `--unsafe-fixes` is set, then [`Applicablility::Suggested`] fixes are included.
+    // - By default, applicable fixes only include [`Applicability::Automatic`], but if
+    //   `--unsafe-fixes` is set, then [`Applicability::Suggested`] fixes are included.
 
     let fix_mode = if cli.diff {
         FixMode::Diff
@@ -289,13 +303,6 @@ pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<Exi
     if show_fixes {
         printer_flags |= PrinterFlags::SHOW_FIX_SUMMARY;
     }
-    if cli.ecosystem_ci {
-        warn_user!(
-            "The formatting of fixes emitted by this option is a work-in-progress, subject to \
-            change at any time, and intended only for internal use."
-        );
-        printer_flags |= PrinterFlags::SHOW_FIX_DIFF;
-    }
 
     #[cfg(debug_assertions)]
     if cache {
@@ -312,7 +319,7 @@ pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<Exi
             commands::add_noqa::add_noqa(&files, &pyproject_config, &config_arguments)?;
         if modifications > 0 && config_arguments.log_level >= LogLevel::Default {
             let s = if modifications == 1 { "" } else { "s" };
-            #[allow(clippy::print_stderr)]
+            #[expect(clippy::print_stderr)]
             {
                 eprintln!("Added {modifications} noqa directive{s}.");
             }
@@ -374,7 +381,7 @@ pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<Exi
         loop {
             match rx.recv() {
                 Ok(event) => {
-                    let Some(change_kind) = change_detected(&event?.paths) else {
+                    let Some(change_kind) = change_detected(&event?) else {
                         continue;
                     };
 
@@ -474,79 +481,119 @@ pub fn check(args: CheckCommand, global_options: GlobalConfigArgs) -> Result<Exi
 mod test_file_change_detector {
     use std::path::PathBuf;
 
-    use crate::{change_detected, ChangeKind};
+    use crate::{ChangeKind, change_detected};
 
     #[test]
     fn detect_correct_file_change() {
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("tmp/pyproject.toml"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/pyproject.toml"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("pyproject.toml"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("pyproject.toml"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("tmp1/tmp2/tmp3/pyproject.toml"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp1/tmp2/tmp3/pyproject.toml"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("tmp/ruff.toml"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/ruff.toml"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("tmp/.ruff.toml"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/.ruff.toml"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::SourceFile),
-            change_detected(&[
-                PathBuf::from("tmp/rule.py"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/rule.py"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::SourceFile),
-            change_detected(&[
-                PathBuf::from("tmp/rule.pyi"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/rule.pyi"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("pyproject.toml"),
-                PathBuf::from("tmp/rule.py"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("pyproject.toml"),
+                    PathBuf::from("tmp/rule.py"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             Some(ChangeKind::Configuration),
-            change_detected(&[
-                PathBuf::from("tmp/rule.py"),
-                PathBuf::from("pyproject.toml"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/rule.py"),
+                    PathBuf::from("pyproject.toml"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
         assert_eq!(
             None,
-            change_detected(&[
-                PathBuf::from("tmp/rule.js"),
-                PathBuf::from("tmp/bin/ruff.rs"),
-            ]),
+            change_detected(&notify::Event {
+                kind: notify::EventKind::Create(notify::event::CreateKind::File),
+                paths: vec![
+                    PathBuf::from("tmp/rule.js"),
+                    PathBuf::from("tmp/bin/ruff.rs"),
+                ],
+                attrs: notify::event::EventAttributes::default(),
+            }),
         );
     }
 }
