@@ -1,25 +1,25 @@
 use std::cmp::Reverse;
-use std::fmt::Display;
 use std::hash::Hash;
 use std::io::Write;
 
 use anyhow::Result;
 use bitflags::bitflags;
 use colored::Colorize;
-use itertools::{iterate, Itertools};
+use itertools::{Itertools, iterate};
+use ruff_linter::codes::NoqaCode;
+use ruff_linter::linter::FixTable;
 use serde::Serialize;
 
 use ruff_linter::fs::relativize_path;
 use ruff_linter::logging::LogLevel;
 use ruff_linter::message::{
     AzureEmitter, Emitter, EmitterContext, GithubEmitter, GitlabEmitter, GroupedEmitter,
-    JsonEmitter, JsonLinesEmitter, JunitEmitter, PylintEmitter, RdjsonEmitter, SarifEmitter,
-    TextEmitter,
+    JsonEmitter, JsonLinesEmitter, JunitEmitter, Message, PylintEmitter, RdjsonEmitter,
+    SarifEmitter, TextEmitter,
 };
 use ruff_linter::notify_user;
-use ruff_linter::registry::{AsRule, Rule};
 use ruff_linter::settings::flags::{self};
-use ruff_linter::settings::types::{SerializationFormat, UnsafeFixes};
+use ruff_linter::settings::types::{OutputFormat, UnsafeFixes};
 
 use crate::diagnostics::{Diagnostics, FixMap};
 
@@ -27,47 +27,24 @@ bitflags! {
     #[derive(Default, Debug, Copy, Clone)]
     pub(crate) struct Flags: u8 {
         /// Whether to show violations when emitting diagnostics.
-        const SHOW_VIOLATIONS = 0b0000_0001;
+        const SHOW_VIOLATIONS = 1 << 0;
         /// Whether to show a summary of the fixed violations when emitting diagnostics.
-        const SHOW_FIX_SUMMARY = 0b0000_0100;
+        const SHOW_FIX_SUMMARY = 1 << 1;
         /// Whether to show a diff of each fixed violation when emitting diagnostics.
-        const SHOW_FIX_DIFF = 0b0000_1000;
+        const SHOW_FIX_DIFF = 1 << 2;
     }
 }
 
 #[derive(Serialize)]
-struct ExpandedStatistics<'a> {
-    code: SerializeRuleAsCode,
-    message: &'a str,
+struct ExpandedStatistics {
+    code: Option<NoqaCode>,
+    name: &'static str,
     count: usize,
     fixable: bool,
 }
 
-struct SerializeRuleAsCode(Rule);
-
-impl Serialize for SerializeRuleAsCode {
-    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
-    where
-        S: serde::Serializer,
-    {
-        serializer.serialize_str(&self.0.noqa_code().to_string())
-    }
-}
-
-impl Display for SerializeRuleAsCode {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.0.noqa_code())
-    }
-}
-
-impl From<Rule> for SerializeRuleAsCode {
-    fn from(rule: Rule) -> Self {
-        Self(rule)
-    }
-}
-
 pub(crate) struct Printer {
-    format: SerializationFormat,
+    format: OutputFormat,
     log_level: LogLevel,
     fix_mode: flags::FixMode,
     unsafe_fixes: UnsafeFixes,
@@ -76,7 +53,7 @@ pub(crate) struct Printer {
 
 impl Printer {
     pub(crate) const fn new(
-        format: SerializationFormat,
+        format: OutputFormat,
         log_level: LogLevel,
         fix_mode: flags::FixMode,
         unsafe_fixes: UnsafeFixes,
@@ -104,7 +81,7 @@ impl Printer {
             let fixed = diagnostics
                 .fixed
                 .values()
-                .flat_map(std::collections::HashMap::values)
+                .flat_map(FixTable::counts)
                 .sum::<usize>();
 
             if self.flags.intersects(Flags::SHOW_VIOLATIONS) {
@@ -127,15 +104,16 @@ impl Printer {
                     let fix_prefix = format!("[{}]", "*".cyan());
 
                     if self.unsafe_fixes.is_hint() {
-                        if fixables.applicable > 0 && fixables.unapplicable_unsafe > 0 {
-                            let es = if fixables.unapplicable_unsafe == 1 {
+                        if fixables.applicable > 0 && fixables.inapplicable_unsafe > 0 {
+                            let es = if fixables.inapplicable_unsafe == 1 {
                                 ""
                             } else {
                                 "es"
                             };
-                            writeln!(writer,
+                            writeln!(
+                                writer,
                                 "{fix_prefix} {} fixable with the `--fix` option ({} hidden fix{es} can be enabled with the `--unsafe-fixes` option).",
-                                fixables.applicable, fixables.unapplicable_unsafe
+                                fixables.applicable, fixables.inapplicable_unsafe
                             )?;
                         } else if fixables.applicable > 0 {
                             // Only applicable fixes
@@ -145,15 +123,16 @@ impl Printer {
                                 fixables.applicable,
                             )?;
                         } else {
-                            // Only unapplicable fixes
-                            let es = if fixables.unapplicable_unsafe == 1 {
+                            // Only inapplicable fixes
+                            let es = if fixables.inapplicable_unsafe == 1 {
                                 ""
                             } else {
                                 "es"
                             };
-                            writeln!(writer,
+                            writeln!(
+                                writer,
                                 "No fixes available ({} hidden fix{es} can be enabled with the `--unsafe-fixes` option).",
-                                fixables.unapplicable_unsafe
+                                fixables.inapplicable_unsafe
                             )?;
                         }
                     } else {
@@ -170,7 +149,7 @@ impl Printer {
                 // Check if there are unapplied fixes
                 let unapplied = {
                     if let Some(fixables) = fixables {
-                        fixables.unapplicable_unsafe
+                        fixables.inapplicable_unsafe
                     } else {
                         0
                     }
@@ -181,15 +160,27 @@ impl Printer {
                     if fixed > 0 {
                         let s = if fixed == 1 { "" } else { "s" };
                         if self.fix_mode.is_apply() {
-                            writeln!(writer, "Fixed {fixed} error{s} ({unapplied} additional fix{es} available with `--unsafe-fixes`).")?;
+                            writeln!(
+                                writer,
+                                "Fixed {fixed} error{s} ({unapplied} additional fix{es} available with `--unsafe-fixes`)."
+                            )?;
                         } else {
-                            writeln!(writer, "Would fix {fixed} error{s} ({unapplied} additional fix{es} available with `--unsafe-fixes`).")?;
+                            writeln!(
+                                writer,
+                                "Would fix {fixed} error{s} ({unapplied} additional fix{es} available with `--unsafe-fixes`)."
+                            )?;
                         }
                     } else {
                         if self.fix_mode.is_apply() {
-                            writeln!(writer, "No errors fixed ({unapplied} fix{es} available with `--unsafe-fixes`).")?;
+                            writeln!(
+                                writer,
+                                "No errors fixed ({unapplied} fix{es} available with `--unsafe-fixes`)."
+                            )?;
                         } else {
-                            writeln!(writer, "No errors would be fixed ({unapplied} fix{es} available with `--unsafe-fixes`).")?;
+                            writeln!(
+                                writer,
+                                "No errors would be fixed ({unapplied} fix{es} available with `--unsafe-fixes`)."
+                            )?;
                         }
                     }
                 } else {
@@ -219,10 +210,7 @@ impl Printer {
         if !self.flags.intersects(Flags::SHOW_VIOLATIONS) {
             if matches!(
                 self.format,
-                SerializationFormat::Text
-                    | SerializationFormat::Full
-                    | SerializationFormat::Concise
-                    | SerializationFormat::Grouped
+                OutputFormat::Full | OutputFormat::Concise | OutputFormat::Grouped
             ) {
                 if self.flags.intersects(Flags::SHOW_FIX_SUMMARY) {
                     if !diagnostics.fixed.is_empty() {
@@ -240,24 +228,23 @@ impl Printer {
         let fixables = FixableStatistics::try_from(diagnostics, self.unsafe_fixes);
 
         match self.format {
-            SerializationFormat::Json => {
+            OutputFormat::Json => {
                 JsonEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Rdjson => {
+            OutputFormat::Rdjson => {
                 RdjsonEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::JsonLines => {
+            OutputFormat::JsonLines => {
                 JsonLinesEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Junit => {
+            OutputFormat::Junit => {
                 JunitEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Concise
-            | SerializationFormat::Full => {
+            OutputFormat::Concise | OutputFormat::Full => {
                 TextEmitter::default()
                     .with_show_fix_status(show_fix_status(self.fix_mode, fixables.as_ref()))
                     .with_show_fix_diff(self.flags.intersects(Flags::SHOW_FIX_DIFF))
-                    .with_show_source(self.format == SerializationFormat::Full)
+                    .with_show_source(self.format == OutputFormat::Full)
                     .with_unsafe_fixes(self.unsafe_fixes)
                     .emit(writer, &diagnostics.messages, &context)?;
 
@@ -271,7 +258,7 @@ impl Printer {
 
                 self.write_summary_text(writer, diagnostics)?;
             }
-            SerializationFormat::Grouped => {
+            OutputFormat::Grouped => {
                 GroupedEmitter::default()
                     .with_show_fix_status(show_fix_status(self.fix_mode, fixables.as_ref()))
                     .with_unsafe_fixes(self.unsafe_fixes)
@@ -286,22 +273,21 @@ impl Printer {
                 }
                 self.write_summary_text(writer, diagnostics)?;
             }
-            SerializationFormat::Github => {
+            OutputFormat::Github => {
                 GithubEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Gitlab => {
+            OutputFormat::Gitlab => {
                 GitlabEmitter::default().emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Pylint => {
+            OutputFormat::Pylint => {
                 PylintEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Azure => {
+            OutputFormat::Azure => {
                 AzureEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Sarif => {
+            OutputFormat::Sarif => {
                 SarifEmitter.emit(writer, &diagnostics.messages, &context)?;
             }
-            SerializationFormat::Text => unreachable!("Text is deprecated and should have been automatically converted to the default serialization format")
         }
 
         writer.flush()?;
@@ -317,30 +303,31 @@ impl Printer {
         let statistics: Vec<ExpandedStatistics> = diagnostics
             .messages
             .iter()
-            .map(|message| {
-                (
-                    message.kind.rule(),
-                    &message.kind.body,
-                    message.fix.is_some(),
-                )
-            })
-            .sorted()
-            .fold(vec![], |mut acc, (rule, body, fixable)| {
-                if let Some((prev_rule, _, _, count)) = acc.last_mut() {
-                    if *prev_rule == rule {
-                        *count += 1;
-                        return acc;
+            .map(|message| (message.noqa_code(), message))
+            .sorted_by_key(|(code, message)| (*code, message.fixable()))
+            .fold(
+                vec![],
+                |mut acc: Vec<((Option<NoqaCode>, &Message), usize)>, (code, message)| {
+                    if let Some(((prev_code, _prev_message), count)) = acc.last_mut() {
+                        if *prev_code == code {
+                            *count += 1;
+                            return acc;
+                        }
                     }
-                }
-                acc.push((rule, body, fixable, 1));
-                acc
-            })
+                    acc.push(((code, message), 1));
+                    acc
+                },
+            )
             .iter()
-            .map(|(rule, message, fixable, count)| ExpandedStatistics {
-                code: (*rule).into(),
-                count: *count,
-                message,
-                fixable: *fixable,
+            .map(|&((code, message), count)| ExpandedStatistics {
+                code,
+                name: message.name(),
+                count,
+                fixable: if let Some(fix) = message.fix() {
+                    fix.applies(self.unsafe_fixes.required_applicability())
+                } else {
+                    false
+                },
             })
             .sorted_by_key(|statistic| Reverse(statistic.count))
             .collect();
@@ -350,9 +337,7 @@ impl Printer {
         }
 
         match self.format {
-            SerializationFormat::Text
-            | SerializationFormat::Full
-            | SerializationFormat::Concise => {
+            OutputFormat::Full | OutputFormat::Concise => {
                 // Compute the maximum number of digits in the count and code, for all messages,
                 // to enable pretty-printing.
                 let count_width = num_digits(
@@ -364,7 +349,12 @@ impl Printer {
                 );
                 let code_width = statistics
                     .iter()
-                    .map(|statistic| statistic.code.to_string().len())
+                    .map(|statistic| {
+                        statistic
+                            .code
+                            .map_or_else(String::new, |rule| rule.to_string())
+                            .len()
+                    })
                     .max()
                     .unwrap();
                 let any_fixable = statistics.iter().any(|statistic| statistic.fixable);
@@ -378,7 +368,11 @@ impl Printer {
                         writer,
                         "{:>count_width$}\t{:<code_width$}\t{}{}",
                         statistic.count.to_string().bold(),
-                        statistic.code.to_string().red().bold(),
+                        statistic
+                            .code
+                            .map_or_else(String::new, |rule| rule.to_string())
+                            .red()
+                            .bold(),
                         if any_fixable {
                             if statistic.fixable {
                                 &fixable
@@ -388,12 +382,14 @@ impl Printer {
                         } else {
                             ""
                         },
-                        statistic.message,
+                        statistic.name,
                     )?;
                 }
+
+                self.write_summary_text(writer, diagnostics)?;
                 return Ok(());
             }
-            SerializationFormat::Json => {
+            OutputFormat::Json => {
                 writeln!(writer, "{}", serde_json::to_string_pretty(&statistics)?)?;
             }
             _ => {
@@ -477,13 +473,13 @@ fn show_fix_status(fix_mode: flags::FixMode, fixables: Option<&FixableStatistics
 fn print_fix_summary(writer: &mut dyn Write, fixed: &FixMap) -> Result<()> {
     let total = fixed
         .values()
-        .map(|table| table.values().sum::<usize>())
+        .map(|table| table.counts().sum::<usize>())
         .sum::<usize>();
     assert!(total > 0);
     let num_digits = num_digits(
-        *fixed
+        fixed
             .values()
-            .filter_map(|table| table.values().max())
+            .filter_map(|table| table.counts().max())
             .max()
             .unwrap(),
     );
@@ -503,12 +499,11 @@ fn print_fix_summary(writer: &mut dyn Write, fixed: &FixMap) -> Result<()> {
             relativize_path(filename).bold(),
             ":".cyan()
         )?;
-        for (rule, count) in table.iter().sorted_by_key(|(.., count)| Reverse(*count)) {
+        for (code, name, count) in table.iter().sorted_by_key(|(.., count)| Reverse(*count)) {
             writeln!(
                 writer,
-                "    {count:>num_digits$} × {} ({})",
-                rule.noqa_code().to_string().red().bold(),
-                rule.as_ref(),
+                "    {count:>num_digits$} × {code} ({name})",
+                code = code.to_string().red().bold(),
             )?;
         }
     }
@@ -519,33 +514,33 @@ fn print_fix_summary(writer: &mut dyn Write, fixed: &FixMap) -> Result<()> {
 #[derive(Debug)]
 struct FixableStatistics {
     applicable: u32,
-    unapplicable_unsafe: u32,
+    inapplicable_unsafe: u32,
 }
 
 impl FixableStatistics {
     fn try_from(diagnostics: &Diagnostics, unsafe_fixes: UnsafeFixes) -> Option<Self> {
         let mut applicable = 0;
-        let mut unapplicable_unsafe = 0;
+        let mut inapplicable_unsafe = 0;
 
         for message in &diagnostics.messages {
-            if let Some(fix) = &message.fix {
+            if let Some(fix) = message.fix() {
                 if fix.applies(unsafe_fixes.required_applicability()) {
                     applicable += 1;
                 } else {
-                    // Do not include unapplicable fixes at other levels that do not provide an opt-in
+                    // Do not include inapplicable fixes at other levels that do not provide an opt-in
                     if fix.applicability().is_unsafe() {
-                        unapplicable_unsafe += 1;
+                        inapplicable_unsafe += 1;
                     }
                 }
             }
         }
 
-        if applicable == 0 && unapplicable_unsafe == 0 {
+        if applicable == 0 && inapplicable_unsafe == 0 {
             None
         } else {
             Some(Self {
                 applicable,
-                unapplicable_unsafe,
+                inapplicable_unsafe,
             })
         }
     }
