@@ -108,3 +108,23 @@ async def import_from_anyio():
 
     # catch from import
     await sleep(86401)  # error: 116, "async"
+
+
+async def test_anyio_async116_helpers():
+    import anyio
+
+    await anyio.sleep(delay=1)  # OK
+    await anyio.sleep(seconds=1)  # OK
+
+    await anyio.sleep(delay=86401)  # ASYNC116
+    await anyio.sleep(seconds=86401)  # OK
+
+
+async def test_trio_async116_helpers():
+    import trio
+
+    await trio.sleep(seconds=1)  # OK
+    await trio.sleep(delay=1)  # OK
+
+    await trio.sleep(seconds=86401)  # ASYNC116
+    await trio.sleep(delay=86401)  # OK
