@@ -1,9 +1,6 @@
 import math
 
-from math import e as special_e
-from math import log as special_log
-
-# Errors.
+# Errors
 math.log(1, 2)
 math.log(1, 10)
 math.log(1, math.e)
@@ -11,15 +8,10 @@ foo = ...
 math.log(foo, 2)
 math.log(foo, 10)
 math.log(foo, math.e)
-math.log(1, special_e)
-special_log(1, 2)
-special_log(1, 10)
-special_log(1, math.e)
-special_log(1, special_e)
 math.log(1, 2.0)
 math.log(1, 10.0)
 
-# Ok.
+# OK
 math.log2(1)
 math.log10(1)
 math.log(1)
@@ -40,6 +32,7 @@ math.log10(1, 2)  # math.log10 takes only one argument.
 
 math.log(1, base=2)  # math.log does not accept keyword arguments.
 
+
 def log(*args):
     print(f"Logging: {args}")
 
@@ -50,3 +43,33 @@ log(1, math.e)
 
 math.log(1, 2.0001)
 math.log(1, 10.0001)
+
+
+# see: https://github.com/astral-sh/ruff/issues/18639
+math.log(1, 10 # comment
+         )
+
+math.log(1,
+         10 # comment
+         )
+
+math.log(1 # comment
+         , # comment
+         10 # comment
+         )
+
+math.log(
+    1 # comment
+    ,
+    10 # comment
+)
+
+math.log(4.13e223, 2)
+math.log(4.14e223, 10)
+
+
+def print_log(*args):
+    try:
+        print(math.log(*args, math.e))
+    except TypeError as e:
+        print(repr(e))
