@@ -5,9 +5,7 @@ use std::path::Path;
 use itertools::Itertools;
 use rustc_hash::FxHashSet;
 
-use ruff_diagnostics::{Diagnostic, Edit, Fix};
 use ruff_python_trivia::CommentRanges;
-use ruff_source_file::Locator;
 use ruff_text_size::Ranged;
 
 use crate::fix::edits::delete_comment;
@@ -20,10 +18,13 @@ use crate::rules::pygrep_hooks;
 use crate::rules::ruff;
 use crate::rules::ruff::rules::{UnusedCodes, UnusedNOQA};
 use crate::settings::LinterSettings;
+use crate::{Edit, Fix, Locator};
 
-#[allow(clippy::too_many_arguments)]
+use super::ast::LintContext;
+
+#[expect(clippy::too_many_arguments)]
 pub(crate) fn check_noqa(
-    diagnostics: &mut Vec<Diagnostic>,
+    context: &mut LintContext,
     path: &Path,
     locator: &Locator,
     comment_ranges: &CommentRanges,
@@ -33,27 +34,27 @@ pub(crate) fn check_noqa(
     settings: &LinterSettings,
 ) -> Vec<usize> {
     // Identify any codes that are globally exempted (within the current file).
-    let file_noqa_directives = FileNoqaDirectives::extract(
-        locator.contents(),
-        comment_ranges,
-        &settings.external,
-        path,
-        locator,
-    );
+    let file_noqa_directives =
+        FileNoqaDirectives::extract(locator, comment_ranges, &settings.external, path);
     let exemption = FileExemption::from(&file_noqa_directives);
 
     // Extract all `noqa` directives.
-    let mut noqa_directives = NoqaDirectives::from_commented_ranges(comment_ranges, path, locator);
+    let mut noqa_directives =
+        NoqaDirectives::from_commented_ranges(comment_ranges, &settings.external, path, locator);
 
     // Indices of diagnostics that were ignored by a `noqa` directive.
     let mut ignored_diagnostics = vec![];
 
     // Remove any ignored diagnostics.
-    'outer: for (index, diagnostic) in diagnostics.iter().enumerate() {
-        if matches!(diagnostic.kind.rule(), Rule::BlanketNOQA) {
+    'outer: for (index, diagnostic) in context.iter().enumerate() {
+        let rule = diagnostic.rule();
+
+        if matches!(rule, Rule::BlanketNOQA) {
             continue;
         }
 
+        let code = rule.noqa_code();
+
         match &exemption {
             FileExemption::All(_) => {
                 // If the file is exempted, ignore all diagnostics.
@@ -62,7 +63,7 @@ pub(crate) fn check_noqa(
             }
             FileExemption::Codes(codes) => {
                 // If the diagnostic is ignored by a global exemption, ignore it.
-                if codes.contains(&&diagnostic.kind.rule().noqa_code()) {
+                if codes.contains(&&code) {
                     ignored_diagnostics.push(index);
                     continue;
                 }
@@ -81,17 +82,13 @@ pub(crate) fn check_noqa(
             {
                 let suppressed = match &directive_line.directive {
                     Directive::All(_) => {
-                        directive_line
-                            .matches
-                            .push(diagnostic.kind.rule().noqa_code());
+                        directive_line.matches.push(code);
                         ignored_diagnostics.push(index);
                         true
                     }
                     Directive::Codes(directive) => {
-                        if directive.includes(diagnostic.kind.rule()) {
-                            directive_line
-                                .matches
-                                .push(diagnostic.kind.rule().noqa_code());
+                        if directive.includes(code) {
+                            directive_line.matches.push(code);
                             ignored_diagnostics.push(index);
                             true
                         } else {
@@ -114,16 +111,24 @@ pub(crate) fn check_noqa(
         && !exemption.includes(Rule::UnusedNOQA)
         && !per_file_ignores.contains(Rule::UnusedNOQA)
     {
-        for line in noqa_directives.lines() {
-            match &line.directive {
+        let directives = noqa_directives
+            .lines()
+            .iter()
+            .map(|line| (&line.directive, &line.matches, false))
+            .chain(
+                file_noqa_directives
+                    .lines()
+                    .iter()
+                    .map(|line| (&line.parsed_file_exemption, &line.matches, true)),
+            );
+        for (directive, matches, is_file_level) in directives {
+            match directive {
                 Directive::All(directive) => {
-                    if line.matches.is_empty() {
-                        let mut diagnostic =
-                            Diagnostic::new(UnusedNOQA { codes: None }, directive.range());
-                        diagnostic
-                            .set_fix(Fix::safe_edit(delete_comment(directive.range(), locator)));
-
-                        diagnostics.push(diagnostic);
+                    if matches.is_empty() {
+                        let edit = delete_comment(directive.range(), locator);
+                        let mut diagnostic = context
+                            .report_diagnostic(UnusedNOQA { codes: None }, directive.range());
+                        diagnostic.set_fix(Fix::safe_edit(edit));
                     }
                 }
                 Directive::Codes(directive) => {
@@ -141,17 +146,19 @@ pub(crate) fn check_noqa(
                             break;
                         }
 
-                        if !seen_codes.insert(original_code) {
-                            duplicated_codes.push(original_code);
-                        } else if line.matches.iter().any(|match_| *match_ == code)
-                            || settings
+                        if seen_codes.insert(original_code) {
+                            let is_code_used = if is_file_level {
+                                context.iter().any(|diag| diag.rule().noqa_code() == code)
+                            } else {
+                                matches.iter().any(|match_| *match_ == code)
+                            } || settings
                                 .external
                                 .iter()
-                                .any(|external| code.starts_with(external))
-                        {
-                            valid_codes.push(original_code);
-                        } else {
-                            if let Ok(rule) = Rule::from_code(code) {
+                                .any(|external| code.starts_with(external));
+
+                            if is_code_used {
+                                valid_codes.push(original_code);
+                            } else if let Ok(rule) = Rule::from_code(code) {
                                 if settings.rules.enabled(rule) {
                                     unmatched_codes.push(original_code);
                                 } else {
@@ -160,6 +167,8 @@ pub(crate) fn check_noqa(
                             } else {
                                 unknown_codes.push(original_code);
                             }
+                        } else {
+                            duplicated_codes.push(original_code);
                         }
                     }
 
@@ -172,7 +181,25 @@ pub(crate) fn check_noqa(
                         && unknown_codes.is_empty()
                         && unmatched_codes.is_empty())
                     {
-                        let mut diagnostic = Diagnostic::new(
+                        let edit = if valid_codes.is_empty() {
+                            delete_comment(directive.range(), locator)
+                        } else {
+                            let original_text = locator.slice(directive.range());
+                            let prefix = if is_file_level {
+                                if original_text.contains("flake8") {
+                                    "# flake8: noqa: "
+                                } else {
+                                    "# ruff: noqa: "
+                                }
+                            } else {
+                                "# noqa: "
+                            };
+                            Edit::range_replacement(
+                                format!("{}{}", prefix, valid_codes.join(", ")),
+                                directive.range(),
+                            )
+                        };
+                        let mut diagnostic = context.report_diagnostic(
                             UnusedNOQA {
                                 codes: Some(UnusedCodes {
                                     disabled: disabled_codes
@@ -195,18 +222,7 @@ pub(crate) fn check_noqa(
                             },
                             directive.range(),
                         );
-                        if valid_codes.is_empty() {
-                            diagnostic.set_fix(Fix::safe_edit(delete_comment(
-                                directive.range(),
-                                locator,
-                            )));
-                        } else {
-                            diagnostic.set_fix(Fix::safe_edit(Edit::range_replacement(
-                                format!("# noqa: {}", valid_codes.join(", ")),
-                                directive.range(),
-                            )));
-                        }
-                        diagnostics.push(diagnostic);
+                        diagnostic.set_fix(Fix::safe_edit(edit));
                     }
                 }
             }
@@ -217,7 +233,8 @@ pub(crate) fn check_noqa(
         && !per_file_ignores.contains(Rule::RedirectedNOQA)
         && !exemption.includes(Rule::RedirectedNOQA)
     {
-        ruff::rules::redirected_noqa(diagnostics, &noqa_directives);
+        ruff::rules::redirected_noqa(context, &noqa_directives);
+        ruff::rules::redirected_file_noqa(context, &file_noqa_directives);
     }
 
     if settings.rules.enabled(Rule::BlanketNOQA)
@@ -225,14 +242,20 @@ pub(crate) fn check_noqa(
         && !exemption.enumerates(Rule::BlanketNOQA)
     {
         pygrep_hooks::rules::blanket_noqa(
-            diagnostics,
+            context,
             &noqa_directives,
             locator,
             &file_noqa_directives,
-            settings.preview,
         );
     }
 
+    if settings.rules.enabled(Rule::InvalidRuleCode)
+        && !per_file_ignores.contains(Rule::InvalidRuleCode)
+        && !exemption.enumerates(Rule::InvalidRuleCode)
+    {
+        ruff::rules::invalid_noqa_code(context, &noqa_directives, locator, &settings.external);
+    }
+
     ignored_diagnostics.sort_unstable();
     ignored_diagnostics
 }
