@@ -1,8 +1,8 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, Expr};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::registry::Rule;
 use crate::rules::flake8_2020::helpers::is_sys;
@@ -37,13 +37,13 @@ use crate::rules::flake8_2020::helpers::is_sys;
 /// ## References
 /// - [Python documentation: `sys.version`](https://docs.python.org/3/library/sys.html#sys.version)
 /// - [Python documentation: `sys.version_info`](https://docs.python.org/3/library/sys.html#sys.version_info)
-#[violation]
-pub struct SysVersionSlice3;
+#[derive(ViolationMetadata)]
+pub(crate) struct SysVersionSlice3;
 
 impl Violation for SysVersionSlice3 {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`sys.version[:3]` referenced (python3.10), use `sys.version_info`")
+        "`sys.version[:3]` referenced (python3.10), use `sys.version_info`".to_string()
     }
 }
 
@@ -77,13 +77,13 @@ impl Violation for SysVersionSlice3 {
 /// ## References
 /// - [Python documentation: `sys.version`](https://docs.python.org/3/library/sys.html#sys.version)
 /// - [Python documentation: `sys.version_info`](https://docs.python.org/3/library/sys.html#sys.version_info)
-#[violation]
-pub struct SysVersion2;
+#[derive(ViolationMetadata)]
+pub(crate) struct SysVersion2;
 
 impl Violation for SysVersion2 {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`sys.version[2]` referenced (python3.10), use `sys.version_info`")
+        "`sys.version[2]` referenced (python3.10), use `sys.version_info`".to_string()
     }
 }
 
@@ -117,13 +117,13 @@ impl Violation for SysVersion2 {
 /// ## References
 /// - [Python documentation: `sys.version`](https://docs.python.org/3/library/sys.html#sys.version)
 /// - [Python documentation: `sys.version_info`](https://docs.python.org/3/library/sys.html#sys.version_info)
-#[violation]
-pub struct SysVersion0;
+#[derive(ViolationMetadata)]
+pub(crate) struct SysVersion0;
 
 impl Violation for SysVersion0 {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`sys.version[0]` referenced (python10), use `sys.version_info`")
+        "`sys.version[0]` referenced (python10), use `sys.version_info`".to_string()
     }
 }
 
@@ -157,18 +157,18 @@ impl Violation for SysVersion0 {
 /// ## References
 /// - [Python documentation: `sys.version`](https://docs.python.org/3/library/sys.html#sys.version)
 /// - [Python documentation: `sys.version_info`](https://docs.python.org/3/library/sys.html#sys.version_info)
-#[violation]
-pub struct SysVersionSlice1;
+#[derive(ViolationMetadata)]
+pub(crate) struct SysVersionSlice1;
 
 impl Violation for SysVersionSlice1 {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`sys.version[:1]` referenced (python10), use `sys.version_info`")
+        "`sys.version[:1]` referenced (python10), use `sys.version_info`".to_string()
     }
 }
 
 /// YTT101, YTT102, YTT301, YTT303
-pub(crate) fn subscript(checker: &mut Checker, value: &Expr, slice: &Expr) {
+pub(crate) fn subscript(checker: &Checker, value: &Expr, slice: &Expr) {
     if is_sys(value, "version", checker.semantic()) {
         match slice {
             Expr::Slice(ast::ExprSlice {
@@ -176,6 +176,7 @@ pub(crate) fn subscript(checker: &mut Checker, value: &Expr, slice: &Expr) {
                 upper: Some(upper),
                 step: None,
                 range: _,
+                node_index: _,
             }) => {
                 if let Expr::NumberLiteral(ast::ExprNumberLiteral {
                     value: ast::Number::Int(i),
@@ -183,13 +184,9 @@ pub(crate) fn subscript(checker: &mut Checker, value: &Expr, slice: &Expr) {
                 }) = upper.as_ref()
                 {
                     if *i == 1 && checker.enabled(Rule::SysVersionSlice1) {
-                        checker
-                            .diagnostics
-                            .push(Diagnostic::new(SysVersionSlice1, value.range()));
+                        checker.report_diagnostic(SysVersionSlice1, value.range());
                     } else if *i == 3 && checker.enabled(Rule::SysVersionSlice3) {
-                        checker
-                            .diagnostics
-                            .push(Diagnostic::new(SysVersionSlice3, value.range()));
+                        checker.report_diagnostic(SysVersionSlice3, value.range());
                     }
                 }
             }
@@ -199,13 +196,9 @@ pub(crate) fn subscript(checker: &mut Checker, value: &Expr, slice: &Expr) {
                 ..
             }) => {
                 if *i == 2 && checker.enabled(Rule::SysVersion2) {
-                    checker
-                        .diagnostics
-                        .push(Diagnostic::new(SysVersion2, value.range()));
+                    checker.report_diagnostic(SysVersion2, value.range());
                 } else if *i == 0 && checker.enabled(Rule::SysVersion0) {
-                    checker
-                        .diagnostics
-                        .push(Diagnostic::new(SysVersion0, value.range()));
+                    checker.report_diagnostic(SysVersion0, value.range());
                 }
             }
 
