@@ -1,12 +1,12 @@
 //! Checks relating to shell injection.
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::helpers::Truthiness;
-use ruff_python_ast::{self as ast, Arguments, Expr, Keyword};
+use ruff_python_ast::{self as ast, Arguments, Expr};
 use ruff_python_semantic::SemanticModel;
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::{
     checkers::ast::Checker, registry::Rule, rules::flake8_bandit::helpers::string_literal,
 };
@@ -36,8 +36,8 @@ use crate::{
 /// ## References
 /// - [Python documentation: `subprocess` — Subprocess management](https://docs.python.org/3/library/subprocess.html)
 /// - [Common Weakness Enumeration: CWE-78](https://cwe.mitre.org/data/definitions/78.html)
-#[violation]
-pub struct SubprocessPopenWithShellEqualsTrue {
+#[derive(ViolationMetadata)]
+pub(crate) struct SubprocessPopenWithShellEqualsTrue {
     safety: Safety,
     is_exact: bool,
 }
@@ -46,14 +46,10 @@ impl Violation for SubprocessPopenWithShellEqualsTrue {
     #[derive_message_formats]
     fn message(&self) -> String {
         match (self.safety, self.is_exact) {
-            (Safety::SeemsSafe, true) => format!(
-                "`subprocess` call with `shell=True` seems safe, but may be changed in the future; consider rewriting without `shell`"
-            ),
-            (Safety::Unknown, true) => format!("`subprocess` call with `shell=True` identified, security issue"),
-            (Safety::SeemsSafe, false) => format!(
-                "`subprocess` call with truthy `shell` seems safe, but may be changed in the future; consider rewriting without `shell`"
-            ),
-            (Safety::Unknown, false) => format!("`subprocess` call with truthy `shell` identified, security issue"),
+            (Safety::SeemsSafe, true) => "`subprocess` call with `shell=True` seems safe, but may be changed in the future; consider rewriting without `shell`".to_string(),
+            (Safety::Unknown, true) => "`subprocess` call with `shell=True` identified, security issue".to_string(),
+            (Safety::SeemsSafe, false) => "`subprocess` call with truthy `shell` seems safe, but may be changed in the future; consider rewriting without `shell`".to_string(),
+            (Safety::Unknown, false) => "`subprocess` call with truthy `shell` identified, security issue".to_string(),
         }
     }
 }
@@ -82,13 +78,13 @@ impl Violation for SubprocessPopenWithShellEqualsTrue {
 /// - [Python documentation: `subprocess` — Subprocess management](https://docs.python.org/3/library/subprocess.html)
 ///
 /// [#4045]: https://github.com/astral-sh/ruff/issues/4045
-#[violation]
-pub struct SubprocessWithoutShellEqualsTrue;
+#[derive(ViolationMetadata)]
+pub(crate) struct SubprocessWithoutShellEqualsTrue;
 
 impl Violation for SubprocessWithoutShellEqualsTrue {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`subprocess` call: check for execution of untrusted input")
+        "`subprocess` call: check for execution of untrusted input".to_string()
     }
 }
 
@@ -120,8 +116,8 @@ impl Violation for SubprocessWithoutShellEqualsTrue {
 ///
 /// ## References
 /// - [Python documentation: Security Considerations](https://docs.python.org/3/library/subprocess.html#security-considerations)
-#[violation]
-pub struct CallWithShellEqualsTrue {
+#[derive(ViolationMetadata)]
+pub(crate) struct CallWithShellEqualsTrue {
     is_exact: bool,
 }
 
@@ -129,9 +125,9 @@ impl Violation for CallWithShellEqualsTrue {
     #[derive_message_formats]
     fn message(&self) -> String {
         if self.is_exact {
-            format!("Function call with `shell=True` parameter identified, security issue")
+            "Function call with `shell=True` parameter identified, security issue".to_string()
         } else {
-            format!("Function call with truthy `shell` parameter identified, security issue")
+            "Function call with truthy `shell` parameter identified, security issue".to_string()
         }
     }
 }
@@ -172,8 +168,8 @@ impl Violation for CallWithShellEqualsTrue {
 ///
 /// ## References
 /// - [Python documentation: `subprocess`](https://docs.python.org/3/library/subprocess.html)
-#[violation]
-pub struct StartProcessWithAShell {
+#[derive(ViolationMetadata)]
+pub(crate) struct StartProcessWithAShell {
     safety: Safety,
 }
 
@@ -181,8 +177,8 @@ impl Violation for StartProcessWithAShell {
     #[derive_message_formats]
     fn message(&self) -> String {
         match self.safety {
-            Safety::SeemsSafe => format!("Starting a process with a shell: seems safe, but may be changed in the future; consider rewriting without `shell`"),
-            Safety::Unknown => format!("Starting a process with a shell, possible injection detected"),
+            Safety::SeemsSafe => "Starting a process with a shell: seems safe, but may be changed in the future; consider rewriting without `shell`".to_string(),
+            Safety::Unknown => "Starting a process with a shell, possible injection detected".to_string(),
         }
     }
 }
@@ -191,29 +187,35 @@ impl Violation for StartProcessWithAShell {
 /// Checks for functions that start a process without a shell.
 ///
 /// ## Why is this bad?
-/// The `subprocess` module provides more powerful facilities for spawning new
-/// processes and retrieving their results; using that module is preferable to
-/// using these functions.
+/// Invoking any kind of external executable via a function call can pose
+/// security risks if arbitrary variables are passed to the executable, or if
+/// the input is otherwise unsanitised or unvalidated.
+///
+/// This rule specifically flags functions in the `os` module that spawn
+/// subprocesses *without* the use of a shell. Note that these typically pose a
+/// much smaller security risk than subprocesses that are started *with* a
+/// shell, which are flagged by [`start-process-with-a-shell`][S605] (`S605`).
+/// This gives you the option of enabling one rule while disabling the other
+/// if you decide that the security risk from these functions is acceptable
+/// for your use case.
 ///
 /// ## Example
 /// ```python
-/// os.spawnlp(os.P_NOWAIT, "/bin/mycmd", "mycmd", "myarg")
+/// import os
+///
+///
+/// def insecure_function(arbitrary_user_input: str):
+///     os.spawnlp(os.P_NOWAIT, "/bin/mycmd", "mycmd", arbitrary_user_input)
 /// ```
 ///
-/// Use instead:
-/// ```python
-/// subprocess.Popen(["/bin/mycmd", "myarg"])
-/// ```
-///
-/// ## References
-/// - [Python documentation: Replacing the `os.spawn` family](https://docs.python.org/3/library/subprocess.html#replacing-the-os-spawn-family)
-#[violation]
-pub struct StartProcessWithNoShell;
+/// [S605]: https://docs.astral.sh/ruff/rules/start-process-with-a-shell
+#[derive(ViolationMetadata)]
+pub(crate) struct StartProcessWithNoShell;
 
 impl Violation for StartProcessWithNoShell {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Starting a process without a shell")
+        "Starting a process without a shell".to_string()
     }
 }
 
@@ -242,13 +244,13 @@ impl Violation for StartProcessWithNoShell {
 /// ## References
 /// - [Python documentation: `subprocess.Popen()`](https://docs.python.org/3/library/subprocess.html#subprocess.Popen)
 /// - [Common Weakness Enumeration: CWE-426](https://cwe.mitre.org/data/definitions/426.html)
-#[violation]
-pub struct StartProcessWithPartialPath;
+#[derive(ViolationMetadata)]
+pub(crate) struct StartProcessWithPartialPath;
 
 impl Violation for StartProcessWithPartialPath {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Starting a process with a partial executable path")
+        "Starting a process with a partial executable path".to_string()
     }
 }
 
@@ -275,18 +277,31 @@ impl Violation for StartProcessWithPartialPath {
 ///
 /// ## References
 /// - [Common Weakness Enumeration: CWE-78](https://cwe.mitre.org/data/definitions/78.html)
-#[violation]
-pub struct UnixCommandWildcardInjection;
+#[derive(ViolationMetadata)]
+pub(crate) struct UnixCommandWildcardInjection;
 
 impl Violation for UnixCommandWildcardInjection {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Possible wildcard injection in call due to `*` usage")
+        "Possible wildcard injection in call due to `*` usage".to_string()
+    }
+}
+
+/// Check if an expression is a trusted input for subprocess.run.
+/// We assume that any str, list[str] or tuple[str] literal can be trusted.
+fn is_trusted_input(arg: &Expr) -> bool {
+    match arg {
+        Expr::StringLiteral(_) => true,
+        Expr::List(ast::ExprList { elts, .. }) | Expr::Tuple(ast::ExprTuple { elts, .. }) => {
+            elts.iter().all(|elt| matches!(elt, Expr::StringLiteral(_)))
+        }
+        Expr::Named(named) => is_trusted_input(&named.value),
+        _ => false,
     }
 }
 
 /// S602, S603, S604, S605, S606, S607, S609
-pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn shell_injection(checker: &Checker, call: &ast::ExprCall) {
     let call_kind = get_call_kind(&call.func, checker.semantic());
     let shell_keyword = find_shell_keyword(&call.arguments, checker.semantic());
 
@@ -296,54 +311,42 @@ pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
                 // S602
                 Some(ShellKeyword {
                     truthiness: truthiness @ (Truthiness::True | Truthiness::Truthy),
-                    keyword,
                 }) => {
                     if checker.enabled(Rule::SubprocessPopenWithShellEqualsTrue) {
-                        checker.diagnostics.push(Diagnostic::new(
+                        checker.report_diagnostic(
                             SubprocessPopenWithShellEqualsTrue {
                                 safety: Safety::from(arg),
                                 is_exact: matches!(truthiness, Truthiness::True),
                             },
-                            keyword.range(),
-                        ));
+                            call.func.range(),
+                        );
                     }
                 }
                 // S603
-                Some(ShellKeyword {
-                    truthiness: Truthiness::False | Truthiness::Falsey | Truthiness::Unknown,
-                    keyword,
-                }) => {
-                    if checker.enabled(Rule::SubprocessWithoutShellEqualsTrue) {
-                        checker.diagnostics.push(Diagnostic::new(
-                            SubprocessWithoutShellEqualsTrue,
-                            keyword.range(),
-                        ));
-                    }
-                }
-                // S603
-                None => {
-                    if checker.enabled(Rule::SubprocessWithoutShellEqualsTrue) {
-                        checker.diagnostics.push(Diagnostic::new(
-                            SubprocessWithoutShellEqualsTrue,
-                            arg.range(),
-                        ));
+                _ => {
+                    if !is_trusted_input(arg) {
+                        if checker.enabled(Rule::SubprocessWithoutShellEqualsTrue) {
+                            checker.report_diagnostic(
+                                SubprocessWithoutShellEqualsTrue,
+                                call.func.range(),
+                            );
+                        }
                     }
                 }
             }
         }
     } else if let Some(ShellKeyword {
         truthiness: truthiness @ (Truthiness::True | Truthiness::Truthy),
-        keyword,
     }) = shell_keyword
     {
         // S604
         if checker.enabled(Rule::CallWithShellEqualsTrue) {
-            checker.diagnostics.push(Diagnostic::new(
+            checker.report_diagnostic(
                 CallWithShellEqualsTrue {
                     is_exact: matches!(truthiness, Truthiness::True),
                 },
-                keyword.range(),
-            ));
+                call.func.range(),
+            );
         }
     }
 
@@ -351,12 +354,12 @@ pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
     if checker.enabled(Rule::StartProcessWithAShell) {
         if matches!(call_kind, Some(CallKind::Shell)) {
             if let Some(arg) = call.arguments.args.first() {
-                checker.diagnostics.push(Diagnostic::new(
+                checker.report_diagnostic(
                     StartProcessWithAShell {
                         safety: Safety::from(arg),
                     },
-                    arg.range(),
-                ));
+                    call.func.range(),
+                );
             }
         }
     }
@@ -364,9 +367,7 @@ pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
     // S606
     if checker.enabled(Rule::StartProcessWithNoShell) {
         if matches!(call_kind, Some(CallKind::NoShell)) {
-            checker
-                .diagnostics
-                .push(Diagnostic::new(StartProcessWithNoShell, call.func.range()));
+            checker.report_diagnostic(StartProcessWithNoShell, call.func.range());
         }
     }
 
@@ -375,9 +376,7 @@ pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
         if call_kind.is_some() {
             if let Some(arg) = call.arguments.args.first() {
                 if is_partial_path(arg) {
-                    checker
-                        .diagnostics
-                        .push(Diagnostic::new(StartProcessWithPartialPath, arg.range()));
+                    checker.report_diagnostic(StartProcessWithPartialPath, arg.range());
                 }
             }
         }
@@ -392,17 +391,13 @@ pub(crate) fn shell_injection(checker: &mut Checker, call: &ast::ExprCall) {
                     Some(CallKind::Subprocess),
                     Some(ShellKeyword {
                         truthiness: Truthiness::True | Truthiness::Truthy,
-                        keyword: _,
                     })
                 )
             )
         {
             if let Some(arg) = call.arguments.args.first() {
                 if is_wildcard_command(arg) {
-                    checker.diagnostics.push(Diagnostic::new(
-                        UnixCommandWildcardInjection,
-                        call.func.range(),
-                    ));
+                    checker.report_diagnostic(UnixCommandWildcardInjection, arg.range());
                 }
             }
         }
@@ -451,21 +446,15 @@ fn get_call_kind(func: &Expr, semantic: &SemanticModel) -> Option<CallKind> {
 }
 
 #[derive(Copy, Clone, Debug)]
-struct ShellKeyword<'a> {
+struct ShellKeyword {
     /// Whether the `shell` keyword argument is set and evaluates to `True`.
     truthiness: Truthiness,
-    /// The `shell` keyword argument.
-    keyword: &'a Keyword,
 }
 
 /// Return the `shell` keyword argument to the given function call, if any.
-fn find_shell_keyword<'a>(
-    arguments: &'a Arguments,
-    semantic: &SemanticModel,
-) -> Option<ShellKeyword<'a>> {
+fn find_shell_keyword(arguments: &Arguments, semantic: &SemanticModel) -> Option<ShellKeyword> {
     arguments.find_keyword("shell").map(|keyword| ShellKeyword {
         truthiness: Truthiness::from_expr(&keyword.value, |id| semantic.has_builtin_binding(id)),
-        keyword,
     })
 }
 
@@ -491,7 +480,7 @@ impl From<&Expr> for Safety {
 ///
 /// ## Examples
 /// ```python
-/// import subprocess
+/// import os
 ///
 /// os.system("/bin/ls")
 /// os.system("./bin/ls")
@@ -541,11 +530,11 @@ fn is_partial_path(expr: &Expr) -> bool {
 /// subprocess.Popen(["/usr/local/bin/rsync", "*", "some_where:"], shell=True)
 /// ```
 fn is_wildcard_command(expr: &Expr) -> bool {
-    if let Expr::List(ast::ExprList { elts, .. }) = expr {
+    if let Expr::List(list) = expr {
         let mut has_star = false;
         let mut has_command = false;
-        for elt in elts {
-            if let Some(text) = string_literal(elt) {
+        for item in list {
+            if let Some(text) = string_literal(item) {
                 has_star |= text.contains('*');
                 has_command |= text.contains("chown")
                     || text.contains("chmod")
