@@ -1,22 +1,24 @@
 use ruff_python_ast::{self as ast, Arguments, Expr, Keyword, Stmt};
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::identifier::Identifier;
-use ruff_python_semantic::analyze::visibility::{is_abstract, is_overload};
 use ruff_python_semantic::SemanticModel;
+use ruff_python_semantic::analyze::visibility::{is_abstract, is_overload};
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::registry::Rule;
 
 /// ## What it does
-/// Checks for abstract classes without abstract methods.
+/// Checks for abstract classes without abstract methods or properties.
+/// Annotated but unassigned class variables are regarded as abstract.
 ///
 /// ## Why is this bad?
 /// Abstract base classes are used to define interfaces. If an abstract base
-/// class has no abstract methods, you may have forgotten to add an abstract
-/// method to the class or omitted an `@abstractmethod` decorator.
+/// class has no abstract methods or properties, you may have forgotten
+/// to add an abstract method or property to the class,
+/// or omitted an `@abstractmethod` decorator.
 ///
 /// If the class is _not_ meant to be used as an interface, consider removing
 /// the `ABC` base class from the class definition.
@@ -24,9 +26,12 @@ use crate::registry::Rule;
 /// ## Example
 /// ```python
 /// from abc import ABC
+/// from typing import ClassVar
 ///
 ///
 /// class Foo(ABC):
+///     class_var: ClassVar[str] = "assigned"
+///
 ///     def method(self):
 ///         bar()
 /// ```
@@ -34,9 +39,12 @@ use crate::registry::Rule;
 /// Use instead:
 /// ```python
 /// from abc import ABC, abstractmethod
+/// from typing import ClassVar
 ///
 ///
 /// class Foo(ABC):
+///     class_var: ClassVar[str]  # unassigned
+///
 ///     @abstractmethod
 ///     def method(self):
 ///         bar()
@@ -44,8 +52,9 @@ use crate::registry::Rule;
 ///
 /// ## References
 /// - [Python documentation: `abc`](https://docs.python.org/3/library/abc.html)
-#[violation]
-pub struct AbstractBaseClassWithoutAbstractMethod {
+/// - [Python documentation: `typing.ClassVar`](https://docs.python.org/3/library/typing.html#typing.ClassVar)
+#[derive(ViolationMetadata)]
+pub(crate) struct AbstractBaseClassWithoutAbstractMethod {
     name: String,
 }
 
@@ -53,7 +62,7 @@ impl Violation for AbstractBaseClassWithoutAbstractMethod {
     #[derive_message_formats]
     fn message(&self) -> String {
         let AbstractBaseClassWithoutAbstractMethod { name } = self;
-        format!("`{name}` is an abstract base class, but it has no abstract methods")
+        format!("`{name}` is an abstract base class, but it has no abstract methods or properties")
     }
 }
 
@@ -67,30 +76,30 @@ impl Violation for AbstractBaseClassWithoutAbstractMethod {
 /// `@abstractmethod` decorator to the method.
 ///
 /// ## Example
+///
 /// ```python
 /// from abc import ABC
 ///
 ///
 /// class Foo(ABC):
-///     def method(self):
-///         ...
+///     def method(self): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// from abc import ABC, abstractmethod
 ///
 ///
 /// class Foo(ABC):
 ///     @abstractmethod
-///     def method(self):
-///         ...
+///     def method(self): ...
 /// ```
 ///
 /// ## References
-/// - [Python documentation: abc](https://docs.python.org/3/library/abc.html)
-#[violation]
-pub struct EmptyMethodWithoutAbstractDecorator {
+/// - [Python documentation: `abc`](https://docs.python.org/3/library/abc.html)
+#[derive(ViolationMetadata)]
+pub(crate) struct EmptyMethodWithoutAbstractDecorator {
     name: String,
 }
 
@@ -122,7 +131,11 @@ fn is_abc_class(bases: &[Expr], keywords: &[Keyword], semantic: &SemanticModel)
 fn is_empty_body(body: &[Stmt]) -> bool {
     body.iter().all(|stmt| match stmt {
         Stmt::Pass(_) => true,
-        Stmt::Expr(ast::StmtExpr { value, range: _ }) => {
+        Stmt::Expr(ast::StmtExpr {
+            value,
+            range: _,
+            node_index: _,
+        }) => {
             matches!(
                 value.as_ref(),
                 Expr::StringLiteral(_) | Expr::EllipsisLiteral(_)
@@ -135,7 +148,7 @@ fn is_empty_body(body: &[Stmt]) -> bool {
 /// B024
 /// B027
 pub(crate) fn abstract_base_class(
-    checker: &mut Checker,
+    checker: &Checker,
     stmt: &Stmt,
     name: &str,
     arguments: Option<&Arguments>,
@@ -187,22 +200,22 @@ pub(crate) fn abstract_base_class(
             && is_empty_body(body)
             && !is_overload(decorator_list, checker.semantic())
         {
-            checker.diagnostics.push(Diagnostic::new(
+            checker.report_diagnostic(
                 EmptyMethodWithoutAbstractDecorator {
                     name: format!("{name}.{method_name}"),
                 },
                 stmt.range(),
-            ));
+            );
         }
     }
     if checker.enabled(Rule::AbstractBaseClassWithoutAbstractMethod) {
         if !has_abstract_method {
-            checker.diagnostics.push(Diagnostic::new(
+            checker.report_diagnostic(
                 AbstractBaseClassWithoutAbstractMethod {
                     name: name.to_string(),
                 },
                 stmt.identifier(),
-            ));
+            );
         }
     }
 }
