@@ -7,6 +7,7 @@ pub(crate) use call_datetime_today::*;
 pub(crate) use call_datetime_utcfromtimestamp::*;
 pub(crate) use call_datetime_utcnow::*;
 pub(crate) use call_datetime_without_tzinfo::*;
+pub(crate) use datetime_min_max::*;
 
 mod call_date_fromtimestamp;
 mod call_date_today;
@@ -17,4 +18,5 @@ mod call_datetime_today;
 mod call_datetime_utcfromtimestamp;
 mod call_datetime_utcnow;
 mod call_datetime_without_tzinfo;
+mod datetime_min_max;
 mod helpers;
