@@ -1,10 +1,10 @@
-use ruff_diagnostics::{Diagnostic, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, helpers::map_subscript};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
-use crate::fix::edits::{add_argument, remove_argument, Parentheses};
+use crate::fix::edits::{Parentheses, add_argument, remove_argument};
+use crate::{Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for classes inheriting from `typing.Generic[]` where `Generic[]` is
@@ -13,8 +13,11 @@ use crate::fix::edits::{add_argument, remove_argument, Parentheses};
 /// ## Why is this bad?
 /// If `Generic[]` is not the final class in the bases tuple, unexpected
 /// behaviour can occur at runtime (See [this CPython issue][1] for an example).
-/// The rule is also applied to stub files, but, unlike at runtime,
-/// in stubs it is purely enforced for stylistic consistency.
+///
+/// The rule is also applied to stub files, where it won't cause issues at
+/// runtime. This is because type checkers may not be able to infer an
+/// accurate [MRO] for the class, which could lead to unexpected or
+/// inaccurate results when they analyze your code.
 ///
 /// For example:
 /// ```python
@@ -43,19 +46,32 @@ use crate::fix::edits::{add_argument, remove_argument, Parentheses};
 /// ):
 ///     ...
 /// ```
+///
+/// ## Fix safety
+///
+/// This rule's fix is always unsafe because reordering base classes can change
+/// the behavior of the code by modifying the class's MRO. The fix will also
+/// delete trailing comments after the `Generic` base class in multi-line base
+/// class lists, if any are present.
+///
+/// ## Fix availability
+///
+/// This rule's fix is only available when there are no `*args` present in the base class list.
+///
 /// ## References
 /// - [`typing.Generic` documentation](https://docs.python.org/3/library/typing.html#typing.Generic)
 ///
 /// [1]: https://github.com/python/cpython/issues/106102
-#[violation]
-pub struct GenericNotLastBaseClass;
+/// [MRO]: https://docs.python.org/3/glossary.html#term-method-resolution-order
+#[derive(ViolationMetadata)]
+pub(crate) struct GenericNotLastBaseClass;
 
 impl Violation for GenericNotLastBaseClass {
     const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
 
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`Generic[]` should always be the last base class")
+        "`Generic[]` should always be the last base class".to_string()
     }
 
     fn fix_title(&self) -> Option<String> {
@@ -64,7 +80,7 @@ impl Violation for GenericNotLastBaseClass {
 }
 
 /// PYI059
-pub(crate) fn generic_not_last_base_class(checker: &mut Checker, class_def: &ast::StmtClassDef) {
+pub(crate) fn generic_not_last_base_class(checker: &Checker, class_def: &ast::StmtClassDef) {
     let Some(bases) = class_def.arguments.as_deref() else {
         return;
     };
@@ -92,14 +108,28 @@ pub(crate) fn generic_not_last_base_class(checker: &mut Checker, class_def: &ast
         return;
     }
 
-    let mut diagnostic = Diagnostic::new(GenericNotLastBaseClass, bases.range());
+    let mut diagnostic = checker.report_diagnostic(GenericNotLastBaseClass, bases.range());
+
+    // Avoid suggesting a fix if any of the arguments is starred. This avoids tricky syntax errors
+    // in cases like
+    //
+    // ```python
+    // class C3(Generic[T], metaclass=type, *[str]): ...
+    // ```
+    //
+    // where we would naively try to put `Generic[T]` after `*[str]`, which is also after a keyword
+    // argument, causing the error.
+    if bases
+        .arguments_source_order()
+        .any(|arg| arg.value().is_starred_expr())
+    {
+        return;
+    }
 
     // No fix if multiple `Generic[]`s are seen in the class bases.
     if generic_base_iter.next().is_none() {
         diagnostic.try_set_fix(|| generate_fix(generic_base, bases, checker));
     }
-
-    checker.diagnostics.push(diagnostic);
 }
 
 fn generate_fix(
@@ -118,5 +148,5 @@ fn generate_fix(
         source,
     );
 
-    Ok(Fix::safe_edits(deletion, [insertion]))
+    Ok(Fix::unsafe_edits(deletion, [insertion]))
 }
