@@ -1,25 +1,26 @@
-use std::fmt;
-
-use ruff_diagnostics::{AlwaysFixableViolation, Violation};
-use ruff_diagnostics::{Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
-use ruff_python_ast::identifier::Identifier;
+use ruff_macros::{ViolationMetadata, derive_message_formats};
+use ruff_python_ast::Decorator;
+use ruff_python_ast::helpers::map_callable;
 use ruff_python_ast::name::UnqualifiedName;
 use ruff_python_ast::visitor;
 use ruff_python_ast::visitor::Visitor;
-use ruff_python_ast::Decorator;
 use ruff_python_ast::{self as ast, Expr, Parameters, Stmt};
-use ruff_python_semantic::analyze::visibility::is_abstract;
 use ruff_python_semantic::SemanticModel;
+use ruff_python_semantic::analyze::visibility::is_abstract;
+use ruff_source_file::LineRanges;
 use ruff_text_size::Ranged;
 use ruff_text_size::{TextLen, TextRange};
+use rustc_hash::FxHashSet;
 
 use crate::checkers::ast::Checker;
 use crate::fix::edits;
 use crate::registry::Rule;
+use crate::{AlwaysFixableViolation, Violation};
+use crate::{Edit, Fix};
 
 use super::helpers::{
-    get_mark_decorators, is_pytest_fixture, is_pytest_yield_fixture, keyword_is_literal,
+    Parentheses, get_mark_decorators, is_pytest_fixture, is_pytest_yield_fixture,
+    keyword_is_literal,
 };
 
 /// ## What it does
@@ -32,26 +33,28 @@ use super::helpers::{
 /// optional.
 ///
 /// Either removing those unnecessary parentheses _or_ requiring them for all
-/// fixtures is fine, but it's best to be consistent.
+/// fixtures is fine, but it's best to be consistent. The rule defaults to
+/// removing unnecessary parentheses, to match the documentation of the
+/// official pytest projects.
 ///
 /// ## Example
-/// ```python
-/// import pytest
 ///
-///
-/// @pytest.fixture
-/// def my_fixture():
-///     ...
-/// ```
-///
-/// Use instead:
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture()
-/// def my_fixture():
-///     ...
+/// def my_fixture(): ...
+/// ```
+///
+/// Use instead:
+///
+/// ```python
+/// import pytest
+///
+///
+/// @pytest.fixture
+/// def my_fixture(): ...
 /// ```
 ///
 /// ## Options
@@ -59,8 +62,8 @@ use super::helpers::{
 ///
 /// ## References
 /// - [`pytest` documentation: API Reference: Fixtures](https://docs.pytest.org/en/latest/reference/reference.html#fixtures-api)
-#[violation]
-pub struct PytestFixtureIncorrectParenthesesStyle {
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestFixtureIncorrectParenthesesStyle {
     expected: Parentheses,
     actual: Parentheses,
 }
@@ -89,29 +92,29 @@ impl AlwaysFixableViolation for PytestFixtureIncorrectParenthesesStyle {
 /// fixture configuration.
 ///
 /// ## Example
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture("module")
-/// def my_fixture():
-///     ...
+/// def my_fixture(): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture(scope="module")
-/// def my_fixture():
-///     ...
+/// def my_fixture(): ...
 /// ```
 ///
 /// ## References
 /// - [`pytest` documentation: `@pytest.fixture` functions](https://docs.pytest.org/en/latest/reference/reference.html#pytest-fixture)
-#[violation]
-pub struct PytestFixturePositionalArgs {
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestFixturePositionalArgs {
     function: String,
 }
 
@@ -130,34 +133,34 @@ impl Violation for PytestFixturePositionalArgs {
 /// `scope="function"` can be omitted, as it is the default.
 ///
 /// ## Example
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture(scope="function")
-/// def my_fixture():
-///     ...
+/// def my_fixture(): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture()
-/// def my_fixture():
-///     ...
+/// def my_fixture(): ...
 /// ```
 ///
 /// ## References
 /// - [`pytest` documentation: `@pytest.fixture` functions](https://docs.pytest.org/en/latest/reference/reference.html#pytest-fixture)
-#[violation]
-pub struct PytestExtraneousScopeFunction;
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestExtraneousScopeFunction;
 
 impl AlwaysFixableViolation for PytestExtraneousScopeFunction {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`scope='function'` is implied in `@pytest.fixture()`")
+        "`scope='function'` is implied in `@pytest.fixture()`".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -165,6 +168,10 @@ impl AlwaysFixableViolation for PytestExtraneousScopeFunction {
     }
 }
 
+/// ## Removal
+/// This rule has been removed because marking fixtures that do not return a value with an underscore
+/// isn't a practice recommended by the pytest community.
+///
 /// ## What it does
 /// Checks for `pytest` fixtures that do not return a value, but are not named
 /// with a leading underscore.
@@ -209,19 +216,25 @@ impl AlwaysFixableViolation for PytestExtraneousScopeFunction {
 ///
 /// ## References
 /// - [`pytest` documentation: `@pytest.fixture` functions](https://docs.pytest.org/en/latest/reference/reference.html#pytest-fixture)
-#[violation]
-pub struct PytestMissingFixtureNameUnderscore {
-    function: String,
-}
+#[derive(ViolationMetadata)]
+#[deprecated(note = "PT004 has been removed")]
+pub(crate) struct PytestMissingFixtureNameUnderscore;
 
+#[expect(deprecated)]
 impl Violation for PytestMissingFixtureNameUnderscore {
-    #[derive_message_formats]
     fn message(&self) -> String {
-        let PytestMissingFixtureNameUnderscore { function } = self;
-        format!("Fixture `{function}` does not return anything, add leading underscore")
+        unreachable!("PT004 has been removed");
+    }
+
+    fn message_formats() -> &'static [&'static str] {
+        &["Fixture `{function}` does not return anything, add leading underscore"]
     }
 }
 
+/// ## Removal
+/// This rule has been removed because marking fixtures that do not return a value with an underscore
+/// isn't a practice recommended by the pytest community.
+///
 /// ## What it does
 /// Checks for `pytest` fixtures that return a value, but are named with a
 /// leading underscore.
@@ -268,16 +281,18 @@ impl Violation for PytestMissingFixtureNameUnderscore {
 ///
 /// ## References
 /// - [`pytest` documentation: `@pytest.fixture` functions](https://docs.pytest.org/en/latest/reference/reference.html#pytest-fixture)
-#[violation]
-pub struct PytestIncorrectFixtureNameUnderscore {
-    function: String,
-}
+#[derive(ViolationMetadata)]
+#[deprecated(note = "PT005 has been removed")]
+pub(crate) struct PytestIncorrectFixtureNameUnderscore;
 
+#[expect(deprecated)]
 impl Violation for PytestIncorrectFixtureNameUnderscore {
-    #[derive_message_formats]
     fn message(&self) -> String {
-        let PytestIncorrectFixtureNameUnderscore { function } = self;
-        format!("Fixture `{function}` returns a value, remove leading underscore")
+        unreachable!("PT005 has been removed");
+    }
+
+    fn message_formats() -> &'static [&'static str] {
+        &["Fixture `{function}` returns a value, remove leading underscore"]
     }
 }
 
@@ -298,38 +313,36 @@ impl Violation for PytestIncorrectFixtureNameUnderscore {
 /// and avoid the confusion caused by unused arguments.
 ///
 /// ## Example
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture
-/// def _patch_something():
-///     ...
+/// def _patch_something(): ...
 ///
 ///
-/// def test_foo(_patch_something):
-///     ...
+/// def test_foo(_patch_something): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.fixture
-/// def _patch_something():
-///     ...
+/// def _patch_something(): ...
 ///
 ///
 /// @pytest.mark.usefixtures("_patch_something")
-/// def test_foo():
-///     ...
+/// def test_foo(): ...
 /// ```
 ///
 /// ## References
 /// - [`pytest` documentation: `pytest.mark.usefixtures`](https://docs.pytest.org/en/latest/reference/reference.html#pytest-mark-usefixtures)
-#[violation]
-pub struct PytestFixtureParamWithoutValue {
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestFixtureParamWithoutValue {
     name: String,
 }
 
@@ -376,13 +389,13 @@ impl Violation for PytestFixtureParamWithoutValue {
 ///
 /// ## References
 /// - [`pytest` documentation: `yield_fixture` functions](https://docs.pytest.org/en/latest/yieldfixture.html)
-#[violation]
-pub struct PytestDeprecatedYieldFixture;
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestDeprecatedYieldFixture;
 
 impl Violation for PytestDeprecatedYieldFixture {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`@pytest.yield_fixture` is deprecated, use `@pytest.fixture`")
+        "`@pytest.yield_fixture` is deprecated, use `@pytest.fixture`".to_string()
     }
 }
 
@@ -435,13 +448,13 @@ impl Violation for PytestDeprecatedYieldFixture {
 /// ## References
 /// - [`pytest` documentation: Adding finalizers directly](https://docs.pytest.org/en/latest/how-to/fixtures.html#adding-finalizers-directly)
 /// - [`pytest` documentation: Factories as fixtures](https://docs.pytest.org/en/latest/how-to/fixtures.html#factories-as-fixtures)
-#[violation]
-pub struct PytestFixtureFinalizerCallback;
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestFixtureFinalizerCallback;
 
 impl Violation for PytestFixtureFinalizerCallback {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Use `yield` instead of `request.addfinalizer`")
+        "Use `yield` instead of `request.addfinalizer`".to_string()
     }
 }
 /// ## What it does
@@ -483,8 +496,8 @@ impl Violation for PytestFixtureFinalizerCallback {
 ///
 /// ## References
 /// - [`pytest` documentation: Teardown/Cleanup](https://docs.pytest.org/en/latest/how-to/fixtures.html#teardown-cleanup-aka-fixture-finalization)
-#[violation]
-pub struct PytestUselessYieldFixture {
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestUselessYieldFixture {
     name: String,
 }
 
@@ -540,13 +553,13 @@ impl AlwaysFixableViolation for PytestUselessYieldFixture {
 ///
 /// ## References
 /// - [`pytest` documentation: `pytest.mark.usefixtures`](https://docs.pytest.org/en/latest/reference/reference.html#pytest-mark-usefixtures)
-#[violation]
-pub struct PytestErroneousUseFixturesOnFixture;
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestErroneousUseFixturesOnFixture;
 
 impl AlwaysFixableViolation for PytestErroneousUseFixturesOnFixture {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`pytest.mark.usefixtures` has no effect on fixtures")
+        "`pytest.mark.usefixtures` has no effect on fixtures".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -582,14 +595,14 @@ impl AlwaysFixableViolation for PytestErroneousUseFixturesOnFixture {
 /// ```
 ///
 /// ## References
-/// - [`pytest-asyncio`](https://pypi.org/project/pytest-asyncio/)
-#[violation]
-pub struct PytestUnnecessaryAsyncioMarkOnFixture;
+/// - [PyPI: `pytest-asyncio`](https://pypi.org/project/pytest-asyncio/)
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestUnnecessaryAsyncioMarkOnFixture;
 
 impl AlwaysFixableViolation for PytestUnnecessaryAsyncioMarkOnFixture {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("`pytest.mark.asyncio` is unnecessary for fixtures")
+        "`pytest.mark.asyncio` is unnecessary for fixtures".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -597,21 +610,6 @@ impl AlwaysFixableViolation for PytestUnnecessaryAsyncioMarkOnFixture {
     }
 }
 
-#[derive(Debug, PartialEq, Eq)]
-enum Parentheses {
-    None,
-    Empty,
-}
-
-impl fmt::Display for Parentheses {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            Parentheses::None => fmt.write_str(""),
-            Parentheses::Empty => fmt.write_str("()"),
-        }
-    }
-}
-
 /// Visitor that skips functions
 #[derive(Debug, Default)]
 struct SkipFunctionsVisitor<'a> {
@@ -624,7 +622,11 @@ struct SkipFunctionsVisitor<'a> {
 impl<'a> Visitor<'a> for SkipFunctionsVisitor<'a> {
     fn visit_stmt(&mut self, stmt: &'a Stmt) {
         match stmt {
-            Stmt::Return(ast::StmtReturn { value, range: _ }) => {
+            Stmt::Return(ast::StmtReturn {
+                value,
+                range: _,
+                node_index: _,
+            }) => {
                 if value.is_some() {
                     self.has_return_with_value = true;
                 }
@@ -639,7 +641,11 @@ impl<'a> Visitor<'a> for SkipFunctionsVisitor<'a> {
             Expr::YieldFrom(_) => {
                 self.has_yield_from = true;
             }
-            Expr::Yield(ast::ExprYield { value, range: _ }) => {
+            Expr::Yield(ast::ExprYield {
+                value,
+                range: _,
+                node_index: _,
+            }) => {
                 self.yield_statements.push(expr);
                 if value.is_some() {
                     self.has_return_with_value = true;
@@ -650,7 +656,7 @@ impl<'a> Visitor<'a> for SkipFunctionsVisitor<'a> {
                     .is_some_and(|name| matches!(name.segments(), ["request", "addfinalizer"]))
                 {
                     self.addfinalizer_call = Some(expr);
-                };
+                }
                 visitor::walk_expr(self, expr);
             }
             _ => {}
@@ -668,27 +674,27 @@ fn fixture_decorator<'a>(
 }
 
 fn pytest_fixture_parentheses(
-    checker: &mut Checker,
+    checker: &Checker,
     decorator: &Decorator,
     fix: Fix,
     expected: Parentheses,
     actual: Parentheses,
 ) {
-    let mut diagnostic = Diagnostic::new(
+    let mut diagnostic = checker.report_diagnostic(
         PytestFixtureIncorrectParenthesesStyle { expected, actual },
         decorator.range(),
     );
     diagnostic.set_fix(fix);
-    checker.diagnostics.push(diagnostic);
 }
 
 /// PT001, PT002, PT003
-fn check_fixture_decorator(checker: &mut Checker, func_name: &str, decorator: &Decorator) {
+fn check_fixture_decorator(checker: &Checker, func_name: &str, decorator: &Decorator) {
     match &decorator.expression {
         Expr::Call(ast::ExprCall {
             func,
             arguments,
             range: _,
+            node_index: _,
         }) => {
             if checker.enabled(Rule::PytestFixtureIncorrectParenthesesStyle) {
                 if !checker.settings.flake8_pytest_style.fixture_parentheses
@@ -708,20 +714,20 @@ fn check_fixture_decorator(checker: &mut Checker, func_name: &str, decorator: &D
 
             if checker.enabled(Rule::PytestFixturePositionalArgs) {
                 if !arguments.args.is_empty() {
-                    checker.diagnostics.push(Diagnostic::new(
+                    checker.report_diagnostic(
                         PytestFixturePositionalArgs {
                             function: func_name.to_string(),
                         },
                         decorator.range(),
-                    ));
+                    );
                 }
             }
 
             if checker.enabled(Rule::PytestExtraneousScopeFunction) {
                 if let Some(keyword) = arguments.find_keyword("scope") {
                     if keyword_is_literal(keyword, "function") {
-                        let mut diagnostic =
-                            Diagnostic::new(PytestExtraneousScopeFunction, keyword.range());
+                        let mut diagnostic = checker
+                            .report_diagnostic(PytestExtraneousScopeFunction, keyword.range());
                         diagnostic.try_set_fix(|| {
                             edits::remove_argument(
                                 keyword,
@@ -731,7 +737,6 @@ fn check_fixture_decorator(checker: &mut Checker, func_name: &str, decorator: &D
                             )
                             .map(Fix::unsafe_edit)
                         });
-                        checker.diagnostics.push(diagnostic);
                     }
                 }
             }
@@ -756,43 +761,14 @@ fn check_fixture_decorator(checker: &mut Checker, func_name: &str, decorator: &D
     }
 }
 
-/// PT004, PT005, PT022
-fn check_fixture_returns(
-    checker: &mut Checker,
-    stmt: &Stmt,
-    name: &str,
-    body: &[Stmt],
-    returns: Option<&Expr>,
-) {
+/// PT022
+fn check_fixture_returns(checker: &Checker, name: &str, body: &[Stmt], returns: Option<&Expr>) {
     let mut visitor = SkipFunctionsVisitor::default();
 
     for stmt in body {
         visitor.visit_stmt(stmt);
     }
 
-    if checker.enabled(Rule::PytestIncorrectFixtureNameUnderscore)
-        && visitor.has_return_with_value
-        && name.starts_with('_')
-    {
-        checker.diagnostics.push(Diagnostic::new(
-            PytestIncorrectFixtureNameUnderscore {
-                function: name.to_string(),
-            },
-            stmt.identifier(),
-        ));
-    } else if checker.enabled(Rule::PytestMissingFixtureNameUnderscore)
-        && !visitor.has_return_with_value
-        && !visitor.has_yield_from
-        && !name.starts_with('_')
-    {
-        checker.diagnostics.push(Diagnostic::new(
-            PytestMissingFixtureNameUnderscore {
-                function: name.to_string(),
-            },
-            stmt.identifier(),
-        ));
-    }
-
     if checker.enabled(Rule::PytestUselessYieldFixture) {
         let Some(stmt) = body.last() else {
             return;
@@ -806,7 +782,7 @@ fn check_fixture_returns(
         if visitor.yield_statements.len() != 1 {
             return;
         }
-        let mut diagnostic = Diagnostic::new(
+        let mut diagnostic = checker.report_diagnostic(
             PytestUselessYieldFixture {
                 name: name.to_string(),
             },
@@ -835,37 +811,74 @@ fn check_fixture_returns(
         } else {
             diagnostic.set_fix(Fix::safe_edit(yield_edit));
         }
-        checker.diagnostics.push(diagnostic);
     }
 }
 
 /// PT019
-fn check_test_function_args(checker: &mut Checker, parameters: &Parameters) {
+fn check_test_function_args(checker: &Checker, parameters: &Parameters, decorators: &[Decorator]) {
+    let mut named_parametrize = FxHashSet::default();
+    for decorator in decorators.iter().filter(|decorator| {
+        UnqualifiedName::from_expr(map_callable(&decorator.expression))
+            .is_some_and(|name| matches!(name.segments(), ["pytest", "mark", "parametrize"]))
+    }) {
+        let Some(call_expr) = decorator.expression.as_call_expr() else {
+            continue;
+        };
+        let Some(first_arg) = call_expr.arguments.find_argument_value("argnames", 0) else {
+            continue;
+        };
+
+        match first_arg {
+            Expr::StringLiteral(ast::ExprStringLiteral { value, .. }) => {
+                named_parametrize.extend(
+                    value
+                        .to_str()
+                        .split(',')
+                        .map(str::trim)
+                        .filter(|param| !param.is_empty() && param.starts_with('_')),
+                );
+            }
+
+            Expr::Name(_) => return,
+            Expr::List(ast::ExprList { elts, .. }) | Expr::Tuple(ast::ExprTuple { elts, .. })
+                if elts.iter().any(Expr::is_name_expr) =>
+            {
+                return;
+            }
+            Expr::List(ast::ExprList { elts, .. }) | Expr::Tuple(ast::ExprTuple { elts, .. }) => {
+                named_parametrize.extend(
+                    elts.iter()
+                        .filter_map(Expr::as_string_literal_expr)
+                        .map(|param| param.value.to_str().trim())
+                        .filter(|param| !param.is_empty() && param.starts_with('_')),
+                );
+            }
+            _ => {}
+        }
+    }
+
     for parameter in parameters.iter_non_variadic_params() {
-        let name = &parameter.parameter.name;
-        if name.starts_with('_') {
-            checker.diagnostics.push(Diagnostic::new(
+        let name = parameter.name();
+        if name.starts_with('_') && !named_parametrize.contains(name.as_str()) {
+            checker.report_diagnostic(
                 PytestFixtureParamWithoutValue {
                     name: name.to_string(),
                 },
                 parameter.range(),
-            ));
+            );
         }
     }
 }
 
 /// PT020
-fn check_fixture_decorator_name(checker: &mut Checker, decorator: &Decorator) {
+fn check_fixture_decorator_name(checker: &Checker, decorator: &Decorator) {
     if is_pytest_yield_fixture(decorator, checker.semantic()) {
-        checker.diagnostics.push(Diagnostic::new(
-            PytestDeprecatedYieldFixture,
-            decorator.range(),
-        ));
+        checker.report_diagnostic(PytestDeprecatedYieldFixture, decorator.range());
     }
 }
 
 /// PT021
-fn check_fixture_addfinalizer(checker: &mut Checker, parameters: &Parameters, body: &[Stmt]) {
+fn check_fixture_addfinalizer(checker: &Checker, parameters: &Parameters, body: &[Stmt]) {
     if !parameters.includes("request") {
         return;
     }
@@ -877,41 +890,35 @@ fn check_fixture_addfinalizer(checker: &mut Checker, parameters: &Parameters, bo
     }
 
     if let Some(addfinalizer) = visitor.addfinalizer_call {
-        checker.diagnostics.push(Diagnostic::new(
-            PytestFixtureFinalizerCallback,
-            addfinalizer.range(),
-        ));
+        checker.report_diagnostic(PytestFixtureFinalizerCallback, addfinalizer.range());
     }
 }
 
 /// PT024, PT025
-fn check_fixture_marks(checker: &mut Checker, decorators: &[Decorator]) {
+fn check_fixture_marks(checker: &Checker, decorators: &[Decorator]) {
     for (expr, marker) in get_mark_decorators(decorators) {
         if checker.enabled(Rule::PytestUnnecessaryAsyncioMarkOnFixture) {
             if marker == "asyncio" {
                 let mut diagnostic =
-                    Diagnostic::new(PytestUnnecessaryAsyncioMarkOnFixture, expr.range());
+                    checker.report_diagnostic(PytestUnnecessaryAsyncioMarkOnFixture, expr.range());
                 let range = checker.locator().full_lines_range(expr.range());
                 diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(range)));
-                checker.diagnostics.push(diagnostic);
             }
         }
 
         if checker.enabled(Rule::PytestErroneousUseFixturesOnFixture) {
             if marker == "usefixtures" {
                 let mut diagnostic =
-                    Diagnostic::new(PytestErroneousUseFixturesOnFixture, expr.range());
+                    checker.report_diagnostic(PytestErroneousUseFixturesOnFixture, expr.range());
                 let line_range = checker.locator().full_lines_range(expr.range());
                 diagnostic.set_fix(Fix::safe_edit(Edit::range_deletion(line_range)));
-                checker.diagnostics.push(diagnostic);
             }
         }
     }
 }
 
 pub(crate) fn fixture(
-    checker: &mut Checker,
-    stmt: &Stmt,
+    checker: &Checker,
     name: &str,
     parameters: &Parameters,
     returns: Option<&Expr>,
@@ -927,18 +934,14 @@ pub(crate) fn fixture(
             check_fixture_decorator(checker, name, decorator);
         }
 
-        if checker.enabled(Rule::PytestDeprecatedYieldFixture)
-            && checker.settings.flake8_pytest_style.fixture_parentheses
-        {
+        if checker.enabled(Rule::PytestDeprecatedYieldFixture) {
             check_fixture_decorator_name(checker, decorator);
         }
 
-        if (checker.enabled(Rule::PytestMissingFixtureNameUnderscore)
-            || checker.enabled(Rule::PytestIncorrectFixtureNameUnderscore)
-            || checker.enabled(Rule::PytestUselessYieldFixture))
+        if checker.enabled(Rule::PytestUselessYieldFixture)
             && !is_abstract(decorators, checker.semantic())
         {
-            check_fixture_returns(checker, stmt, name, body, returns);
+            check_fixture_returns(checker, name, body, returns);
         }
 
         if checker.enabled(Rule::PytestFixtureFinalizerCallback) {
@@ -953,6 +956,6 @@ pub(crate) fn fixture(
     }
 
     if checker.enabled(Rule::PytestFixtureParamWithoutValue) && name.starts_with("test_") {
-        check_test_function_args(checker, parameters);
+        check_test_function_args(checker, parameters, decorators);
     }
 }
