@@ -1,21 +1,21 @@
 use ruff_python_ast::{self as ast, Arguments, Decorator, Expr};
 
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
 use crate::registry::Rule;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
-use super::helpers::get_mark_decorators;
+use super::helpers::{Parentheses, get_mark_decorators};
 
 /// ## What it does
 /// Checks for argument-free `@pytest.mark.<marker>()` decorators with or
 /// without parentheses, depending on the [`lint.flake8-pytest-style.mark-parentheses`]
 /// setting.
 ///
-/// In [preview], this rule defaults to removing unnecessary parentheses, to match
-/// the behavior of official pytest projects.
+/// The rule defaults to removing unnecessary parentheses,
+/// to match the documentation of the official pytest projects.
 ///
 /// ## Why is this bad?
 /// If a `@pytest.mark.<marker>()` doesn't take any arguments, the parentheses are
@@ -25,23 +25,23 @@ use super::helpers::get_mark_decorators;
 /// fixtures is fine, but it's best to be consistent.
 ///
 /// ## Example
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.mark.foo
-/// def test_something():
-///     ...
+/// def test_something(): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.mark.foo()
-/// def test_something():
-///     ...
+/// def test_something(): ...
 /// ```
 ///
 /// ## Options
@@ -49,13 +49,11 @@ use super::helpers::get_mark_decorators;
 ///
 /// ## References
 /// - [`pytest` documentation: Marks](https://docs.pytest.org/en/latest/reference/reference.html#marks)
-///
-/// [preview]: https://docs.astral.sh/ruff/preview/
-#[violation]
-pub struct PytestIncorrectMarkParenthesesStyle {
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestIncorrectMarkParenthesesStyle {
     mark_name: String,
-    expected_parens: String,
-    actual_parens: String,
+    expected_parens: Parentheses,
+    actual_parens: Parentheses,
 }
 
 impl AlwaysFixableViolation for PytestIncorrectMarkParenthesesStyle {
@@ -73,7 +71,10 @@ impl AlwaysFixableViolation for PytestIncorrectMarkParenthesesStyle {
     }
 
     fn fix_title(&self) -> String {
-        "Add/remove parentheses".to_string()
+        match &self.expected_parens {
+            Parentheses::None => "Remove parentheses".to_string(),
+            Parentheses::Empty => "Add parentheses".to_string(),
+        }
     }
 }
 
@@ -86,31 +87,30 @@ impl AlwaysFixableViolation for PytestIncorrectMarkParenthesesStyle {
 /// useless and should be removed.
 ///
 /// ## Example
+///
 /// ```python
 /// import pytest
 ///
 ///
 /// @pytest.mark.usefixtures()
-/// def test_something():
-///     ...
+/// def test_something(): ...
 /// ```
 ///
 /// Use instead:
+///
 /// ```python
-/// def test_something():
-///     ...
+/// def test_something(): ...
 /// ```
 ///
 /// ## References
 /// - [`pytest` documentation: `pytest.mark.usefixtures`](https://docs.pytest.org/en/latest/reference/reference.html#pytest-mark-usefixtures)
-
-#[violation]
-pub struct PytestUseFixturesWithoutParameters;
+#[derive(ViolationMetadata)]
+pub(crate) struct PytestUseFixturesWithoutParameters;
 
 impl AlwaysFixableViolation for PytestUseFixturesWithoutParameters {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Useless `pytest.mark.usefixtures` without parameters")
+        "Useless `pytest.mark.usefixtures` without parameters".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -119,26 +119,25 @@ impl AlwaysFixableViolation for PytestUseFixturesWithoutParameters {
 }
 
 fn pytest_mark_parentheses(
-    checker: &mut Checker,
+    checker: &Checker,
     decorator: &Decorator,
     marker: &str,
     fix: Fix,
-    preferred: &str,
-    actual: &str,
+    preferred: Parentheses,
+    actual: Parentheses,
 ) {
-    let mut diagnostic = Diagnostic::new(
+    let mut diagnostic = checker.report_diagnostic(
         PytestIncorrectMarkParenthesesStyle {
             mark_name: marker.to_string(),
-            expected_parens: preferred.to_string(),
-            actual_parens: actual.to_string(),
+            expected_parens: preferred,
+            actual_parens: actual,
         },
         decorator.range(),
     );
     diagnostic.set_fix(fix);
-    checker.diagnostics.push(diagnostic);
 }
 
-fn check_mark_parentheses(checker: &mut Checker, decorator: &Decorator, marker: &str) {
+fn check_mark_parentheses(checker: &Checker, decorator: &Decorator, marker: &str) {
     match &decorator.expression {
         Expr::Call(ast::ExprCall {
             func,
@@ -147,27 +146,46 @@ fn check_mark_parentheses(checker: &mut Checker, decorator: &Decorator, marker:
                     args,
                     keywords,
                     range: _,
+                    node_index: _,
                 },
             range: _,
+            node_index: _,
         }) => {
             if !checker.settings.flake8_pytest_style.mark_parentheses
                 && args.is_empty()
                 && keywords.is_empty()
             {
                 let fix = Fix::safe_edit(Edit::deletion(func.end(), decorator.end()));
-                pytest_mark_parentheses(checker, decorator, marker, fix, "", "()");
+                pytest_mark_parentheses(
+                    checker,
+                    decorator,
+                    marker,
+                    fix,
+                    Parentheses::None,
+                    Parentheses::Empty,
+                );
             }
         }
         _ => {
             if checker.settings.flake8_pytest_style.mark_parentheses {
-                let fix = Fix::safe_edit(Edit::insertion("()".to_string(), decorator.end()));
-                pytest_mark_parentheses(checker, decorator, marker, fix, "()", "");
+                let fix = Fix::safe_edit(Edit::insertion(
+                    Parentheses::Empty.to_string(),
+                    decorator.end(),
+                ));
+                pytest_mark_parentheses(
+                    checker,
+                    decorator,
+                    marker,
+                    fix,
+                    Parentheses::Empty,
+                    Parentheses::None,
+                );
             }
         }
     }
 }
 
-fn check_useless_usefixtures(checker: &mut Checker, decorator: &Decorator, marker: &str) {
+fn check_useless_usefixtures(checker: &Checker, decorator: &Decorator, marker: &str) {
     if marker != "usefixtures" {
         return;
     }
@@ -187,12 +205,12 @@ fn check_useless_usefixtures(checker: &mut Checker, decorator: &Decorator, marke
         _ => return,
     }
 
-    let mut diagnostic = Diagnostic::new(PytestUseFixturesWithoutParameters, decorator.range());
+    let mut diagnostic =
+        checker.report_diagnostic(PytestUseFixturesWithoutParameters, decorator.range());
     diagnostic.set_fix(Fix::unsafe_edit(Edit::range_deletion(decorator.range())));
-    checker.diagnostics.push(diagnostic);
 }
 
-pub(crate) fn marks(checker: &mut Checker, decorators: &[Decorator]) {
+pub(crate) fn marks(checker: &Checker, decorators: &[Decorator]) {
     let enforce_parentheses = checker.enabled(Rule::PytestIncorrectMarkParenthesesStyle);
     let enforce_useless_usefixtures = checker.enabled(Rule::PytestUseFixturesWithoutParameters);
 
