@@ -1,13 +1,14 @@
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_notebook::CellOffsets;
 use ruff_python_ast::PySourceType;
-use ruff_python_parser::{TokenKind, TokenKindIter};
-use ruff_text_size::{TextRange, TextSize};
-
-use ruff_diagnostics::{AlwaysFixableViolation, Violation};
-use ruff_diagnostics::{Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
 use ruff_python_index::Indexer;
-use ruff_source_file::Locator;
+use ruff_python_parser::{TokenIterWithContext, TokenKind, Tokens};
+use ruff_text_size::{Ranged, TextSize};
+
+use crate::Locator;
+use crate::checkers::ast::LintContext;
+use crate::{AlwaysFixableViolation, Violation};
+use crate::{Edit, Fix};
 
 /// ## What it does
 /// Checks for compound statements (multiple statements on the same line).
@@ -27,13 +28,13 @@ use ruff_source_file::Locator;
 /// ```
 ///
 /// [PEP 8]: https://peps.python.org/pep-0008/#other-recommendations
-#[violation]
-pub struct MultipleStatementsOnOneLineColon;
+#[derive(ViolationMetadata)]
+pub(crate) struct MultipleStatementsOnOneLineColon;
 
 impl Violation for MultipleStatementsOnOneLineColon {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Multiple statements on one line (colon)")
+        "Multiple statements on one line (colon)".to_string()
     }
 }
 
@@ -57,13 +58,13 @@ impl Violation for MultipleStatementsOnOneLineColon {
 /// ```
 ///
 /// [PEP 8]: https://peps.python.org/pep-0008/#other-recommendations
-#[violation]
-pub struct MultipleStatementsOnOneLineSemicolon;
+#[derive(ViolationMetadata)]
+pub(crate) struct MultipleStatementsOnOneLineSemicolon;
 
 impl Violation for MultipleStatementsOnOneLineSemicolon {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Multiple statements on one line (semicolon)")
+        "Multiple statements on one line (semicolon)".to_string()
     }
 }
 
@@ -82,24 +83,24 @@ impl Violation for MultipleStatementsOnOneLineSemicolon {
 /// ```python
 /// do_four()
 /// ```
-#[violation]
-pub struct UselessSemicolon;
+#[derive(ViolationMetadata)]
+pub(crate) struct UselessSemicolon;
 
 impl AlwaysFixableViolation for UselessSemicolon {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Statement ends with an unnecessary semicolon")
+        "Statement ends with an unnecessary semicolon".to_string()
     }
 
     fn fix_title(&self) -> String {
-        format!("Remove unnecessary semicolon")
+        "Remove unnecessary semicolon".to_string()
     }
 }
 
 /// E701, E702, E703
 pub(crate) fn compound_statements(
-    diagnostics: &mut Vec<Diagnostic>,
-    mut tokens: TokenKindIter,
+    context: &LintContext,
+    tokens: &Tokens,
     locator: &Locator,
     indexer: &Indexer,
     source_type: PySourceType,
@@ -125,34 +126,18 @@ pub(crate) fn compound_statements(
     // This is used to allow `class C: ...`-style definitions in stubs.
     let mut allow_ellipsis = false;
 
-    // Track the bracket depth.
-    let mut par_count = 0u32;
-    let mut sqb_count = 0u32;
-    let mut brace_count = 0u32;
-
     // Track indentation.
     let mut indent = 0u32;
 
-    while let Some((token, range)) = tokens.next() {
-        match token {
-            TokenKind::Lpar => {
-                par_count = par_count.saturating_add(1);
-            }
-            TokenKind::Rpar => {
-                par_count = par_count.saturating_sub(1);
-            }
-            TokenKind::Lsqb => {
-                sqb_count = sqb_count.saturating_add(1);
-            }
-            TokenKind::Rsqb => {
-                sqb_count = sqb_count.saturating_sub(1);
-            }
-            TokenKind::Lbrace => {
-                brace_count = brace_count.saturating_add(1);
-            }
-            TokenKind::Rbrace => {
-                brace_count = brace_count.saturating_sub(1);
-            }
+    // Use an iterator to allow passing it around.
+    let mut token_iter = tokens.iter_with_context();
+
+    loop {
+        let Some(token) = token_iter.next() else {
+            break;
+        };
+
+        match token.kind() {
             TokenKind::Ellipsis => {
                 if allow_ellipsis {
                     allow_ellipsis = false;
@@ -168,30 +153,29 @@ pub(crate) fn compound_statements(
             _ => {}
         }
 
-        if par_count > 0 || sqb_count > 0 || brace_count > 0 {
+        if token_iter.in_parenthesized_context() {
             continue;
         }
 
-        match token {
+        match token.kind() {
             TokenKind::Newline => {
-                if let Some((start, end)) = semi {
+                if let Some(range) = semi {
                     if !(source_type.is_ipynb()
                         && indent == 0
                         && cell_offsets
-                            .and_then(|cell_offsets| cell_offsets.containing_range(range.start()))
+                            .and_then(|cell_offsets| cell_offsets.containing_range(token.start()))
                             .is_some_and(|cell_range| {
-                                !has_non_trivia_tokens_till(tokens.clone(), cell_range.end())
+                                !has_non_trivia_tokens_till(token_iter.clone(), cell_range.end())
                             }))
                     {
-                        let mut diagnostic =
-                            Diagnostic::new(UselessSemicolon, TextRange::new(start, end));
-                        diagnostic.set_fix(Fix::safe_edit(Edit::deletion(
-                            indexer
-                                .preceded_by_continuations(start, locator)
-                                .unwrap_or(start),
-                            end,
-                        )));
-                        diagnostics.push(diagnostic);
+                        context
+                            .report_diagnostic(UselessSemicolon, range)
+                            .set_fix(Fix::safe_edit(Edit::deletion(
+                                indexer
+                                    .preceded_by_continuations(range.start(), locator.contents())
+                                    .unwrap_or(range.start()),
+                                range.end(),
+                            )));
                     }
                 }
 
@@ -225,14 +209,14 @@ pub(crate) fn compound_statements(
                     || while_.is_some()
                     || with.is_some()
                 {
-                    colon = Some((range.start(), range.end()));
+                    colon = Some(token.range());
 
                     // Allow `class C: ...`-style definitions.
                     allow_ellipsis = true;
                 }
             }
             TokenKind::Semi => {
-                semi = Some((range.start(), range.end()));
+                semi = Some(token.range());
                 allow_ellipsis = false;
             }
             TokenKind::Comment
@@ -240,22 +224,16 @@ pub(crate) fn compound_statements(
             | TokenKind::Dedent
             | TokenKind::NonLogicalNewline => {}
             _ => {
-                if let Some((start, end)) = semi {
-                    diagnostics.push(Diagnostic::new(
-                        MultipleStatementsOnOneLineSemicolon,
-                        TextRange::new(start, end),
-                    ));
+                if let Some(range) = semi {
+                    context.report_diagnostic(MultipleStatementsOnOneLineSemicolon, range);
 
                     // Reset.
                     semi = None;
                     allow_ellipsis = false;
                 }
 
-                if let Some((start, end)) = colon {
-                    diagnostics.push(Diagnostic::new(
-                        MultipleStatementsOnOneLineColon,
-                        TextRange::new(start, end),
-                    ));
+                if let Some(range) = colon {
+                    context.report_diagnostic(MultipleStatementsOnOneLineColon, range);
 
                     // Reset.
                     colon = None;
@@ -276,7 +254,7 @@ pub(crate) fn compound_statements(
             }
         }
 
-        match token {
+        match token.kind() {
             TokenKind::Lambda => {
                 // Reset.
                 colon = None;
@@ -294,55 +272,55 @@ pub(crate) fn compound_statements(
                 with = None;
             }
             TokenKind::Case => {
-                case = Some((range.start(), range.end()));
+                case = Some(token.range());
             }
             TokenKind::If => {
-                if_ = Some((range.start(), range.end()));
+                if_ = Some(token.range());
             }
             TokenKind::While => {
-                while_ = Some((range.start(), range.end()));
+                while_ = Some(token.range());
             }
             TokenKind::For => {
-                for_ = Some((range.start(), range.end()));
+                for_ = Some(token.range());
             }
             TokenKind::Try => {
-                try_ = Some((range.start(), range.end()));
+                try_ = Some(token.range());
             }
             TokenKind::Except => {
-                except = Some((range.start(), range.end()));
+                except = Some(token.range());
             }
             TokenKind::Finally => {
-                finally = Some((range.start(), range.end()));
+                finally = Some(token.range());
             }
             TokenKind::Elif => {
-                elif = Some((range.start(), range.end()));
+                elif = Some(token.range());
             }
             TokenKind::Else => {
-                else_ = Some((range.start(), range.end()));
+                else_ = Some(token.range());
             }
             TokenKind::Class => {
-                class = Some((range.start(), range.end()));
+                class = Some(token.range());
             }
             TokenKind::With => {
-                with = Some((range.start(), range.end()));
+                with = Some(token.range());
             }
             TokenKind::Match => {
-                match_ = Some((range.start(), range.end()));
+                match_ = Some(token.range());
             }
             _ => {}
-        };
+        }
     }
 }
 
 /// Returns `true` if there are any non-trivia tokens from the given token
 /// iterator till the given end offset.
-fn has_non_trivia_tokens_till(tokens: TokenKindIter, cell_end: TextSize) -> bool {
-    for (token, tok_range) in tokens {
-        if tok_range.start() >= cell_end {
+fn has_non_trivia_tokens_till(token_iter: TokenIterWithContext<'_>, cell_end: TextSize) -> bool {
+    for token in token_iter {
+        if token.start() >= cell_end {
             return false;
         }
         if !matches!(
-            token,
+            token.kind(),
             TokenKind::Newline
                 | TokenKind::Comment
                 | TokenKind::EndOfFile
