@@ -1,10 +1,10 @@
 use ruff_python_ast::{self as ast, Expr};
 
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::name::Name;
 use ruff_text_size::Ranged;
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 
 /// ## What it does
@@ -28,8 +28,8 @@ use crate::checkers::ast::Checker;
 /// print(a)  # 3
 /// ```
 ///
-#[violation]
-pub struct RedeclaredAssignedName {
+#[derive(ViolationMetadata)]
+pub(crate) struct RedeclaredAssignedName {
     name: String,
 }
 
@@ -42,7 +42,7 @@ impl Violation for RedeclaredAssignedName {
 }
 
 /// PLW0128
-pub(crate) fn redeclared_assigned_name(checker: &mut Checker, targets: &Vec<Expr>) {
+pub(crate) fn redeclared_assigned_name(checker: &Checker, targets: &Vec<Expr>) {
     let mut names: Vec<Name> = Vec::new();
 
     for target in targets {
@@ -50,25 +50,33 @@ pub(crate) fn redeclared_assigned_name(checker: &mut Checker, targets: &Vec<Expr
     }
 }
 
-fn check_expr(checker: &mut Checker, expr: &Expr, names: &mut Vec<Name>) {
+fn check_expr(checker: &Checker, expr: &Expr, names: &mut Vec<Name>) {
     match expr {
-        Expr::Tuple(ast::ExprTuple { elts, .. }) => {
-            for target in elts {
+        Expr::Tuple(tuple) => {
+            for target in tuple {
                 check_expr(checker, target, names);
             }
         }
+        Expr::List(list) => {
+            for target in list {
+                check_expr(checker, target, names);
+            }
+        }
+        Expr::Starred(starred) => {
+            check_expr(checker, &starred.value, names);
+        }
         Expr::Name(ast::ExprName { id, .. }) => {
             if checker.settings.dummy_variable_rgx.is_match(id) {
                 // Ignore dummy variable assignments
                 return;
             }
             if names.contains(id) {
-                checker.diagnostics.push(Diagnostic::new(
+                checker.report_diagnostic(
                     RedeclaredAssignedName {
                         name: id.to_string(),
                     },
                     expr.range(),
-                ));
+                );
             }
             names.push(id.clone());
         }
