@@ -1,10 +1,10 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Applicability, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::visitor::Visitor;
-use ruff_python_ast::{self as ast, visitor, Expr, ExprLambda, Parameter, ParameterWithDefault};
+use ruff_python_ast::{self as ast, Expr, ExprLambda, Parameter, ParameterWithDefault, visitor};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
+use crate::{AlwaysFixableViolation, Applicability, Edit, Fix};
 
 /// ## What it does
 /// Checks for `lambda` definitions that consist of a single function call
@@ -43,13 +43,13 @@ use crate::checkers::ast::Checker;
 /// breaking change for callers that execute the lambda by passing arguments by
 /// name, as in: `foo(x=1, y=2)`. Since `func` does not define the arguments
 /// `x` and `y`, unlike the lambda, the call would raise a `TypeError`.
-#[violation]
-pub struct UnnecessaryLambda;
+#[derive(ViolationMetadata)]
+pub(crate) struct UnnecessaryLambda;
 
 impl AlwaysFixableViolation for UnnecessaryLambda {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Lambda may be unnecessary; consider inlining inner function")
+        "Lambda may be unnecessary; consider inlining inner function".to_string()
     }
 
     fn fix_title(&self) -> String {
@@ -58,11 +58,12 @@ impl AlwaysFixableViolation for UnnecessaryLambda {
 }
 
 /// PLW0108
-pub(crate) fn unnecessary_lambda(checker: &mut Checker, lambda: &ExprLambda) {
+pub(crate) fn unnecessary_lambda(checker: &Checker, lambda: &ExprLambda) {
     let ExprLambda {
         parameters,
         body,
         range: _,
+        node_index: _,
     } = lambda;
 
     // The lambda should consist of a single function call.
@@ -179,7 +180,7 @@ pub(crate) fn unnecessary_lambda(checker: &mut Checker, lambda: &ExprLambda) {
             if call_posargs.len() != lambda_posargs.len() {
                 return;
             }
-            for (param, arg) in lambda_posargs.iter().zip(call_posargs.iter()) {
+            for (param, arg) in lambda_posargs.iter().zip(call_posargs) {
                 let Expr::Name(ast::ExprName { id, .. }) = arg else {
                     return;
                 };
@@ -207,7 +208,7 @@ pub(crate) fn unnecessary_lambda(checker: &mut Checker, lambda: &ExprLambda) {
         }
     }
 
-    let mut diagnostic = Diagnostic::new(UnnecessaryLambda, lambda.range());
+    let mut diagnostic = checker.report_diagnostic(UnnecessaryLambda, lambda.range());
     diagnostic.set_fix(Fix::applicable_edit(
         Edit::range_replacement(
             checker.locator().slice(func.as_ref()).to_string(),
@@ -215,7 +216,6 @@ pub(crate) fn unnecessary_lambda(checker: &mut Checker, lambda: &ExprLambda) {
         ),
         Applicability::Unsafe,
     ));
-    checker.diagnostics.push(diagnostic);
 }
 
 /// Identify all `Expr::Name` nodes in an AST.
