@@ -1,18 +1,17 @@
 use itertools::Itertools;
-use ruff_python_ast::{Alias, StmtImportFrom};
 
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::whitespace::indentation;
+use ruff_python_ast::{Alias, StmtImportFrom};
 use ruff_python_codegen::Stylist;
 use ruff_python_parser::Tokens;
-use ruff_source_file::Locator;
 use ruff_text_size::Ranged;
 
+use crate::Locator;
 use crate::checkers::ast::Checker;
-
 use crate::rules::pyupgrade::fixes;
-use crate::settings::types::PythonVersion;
+use crate::{Edit, Fix, FixAvailability, Violation};
+use ruff_python_ast::PythonVersion;
 
 /// An import was moved and renamed as part of a deprecation.
 /// For example, `typing.AbstractSet` was moved to `collections.abc.Set`.
@@ -61,8 +60,8 @@ enum Deprecation {
 /// ```python
 /// from collections.abc import Sequence
 /// ```
-#[violation]
-pub struct DeprecatedImport {
+#[derive(ViolationMetadata)]
+pub(crate) struct DeprecatedImport {
     deprecation: Deprecation,
 }
 
@@ -272,7 +271,7 @@ const TYPING_TO_RE_39: &[&str] = &["Match", "Pattern"];
 const TYPING_RE_TO_RE_39: &[&str] = &["Match", "Pattern"];
 
 // Members of `typing_extensions` that were moved to `typing`.
-const TYPING_EXTENSIONS_TO_TYPING_39: &[&str] = &["Annotated", "get_type_hints"];
+const TYPING_EXTENSIONS_TO_TYPING_39: &[&str] = &["Annotated"];
 
 // Members of `typing` that were moved _and_ renamed (and thus cannot be
 // automatically fixed).
@@ -309,7 +308,9 @@ const TYPING_EXTENSIONS_TO_TYPING_310: &[&str] = &[
     "TypeGuard",
     "get_args",
     "get_origin",
-    "is_typeddict",
+    // Introduced in Python 3.10, but `typing_extensions` equivalent
+    // also checks for `typing_extensions.TypedDict` in addition to `typing.TypedDict`.
+    // "is_typeddict",
 ];
 
 // Python 3.11+
@@ -372,6 +373,9 @@ const TYPING_EXTENSIONS_TO_TYPING_313: &[&str] = &[
     "NoDefault",
     "ReadOnly",
     "TypeIs",
+    // Introduced in Python 3.5,
+    // but typing_extensions backports features from py313:
+    "get_type_hints",
     // Introduced in Python 3.6,
     // but typing_extensions backports features from py313:
     "ContextManager",
@@ -383,13 +387,14 @@ const TYPING_EXTENSIONS_TO_TYPING_313: &[&str] = &[
     // Introduced in Python 3.8, but typing_extensions
     // backports features and bugfixes from py313:
     "Protocol",
-    "TypedDict",
     "runtime_checkable",
     // Introduced in earlier Python versions,
     // but typing_extensions backports PEP-696:
     "ParamSpec",
     "TypeVar",
     "TypeVarTuple",
+    // `typing_extensions` backports PEP 728 (TypedDict with Typed Extra Items)
+    // "TypedDict",
 ];
 
 // Members of `typing_extensions` that were moved to `types`.
@@ -430,7 +435,7 @@ impl<'a> ImportReplacer<'a> {
     fn with_renames(&self) -> Vec<WithRename> {
         let mut operations = vec![];
         if self.module == "typing" {
-            if self.version >= PythonVersion::Py39 {
+            if self.version >= PythonVersion::PY39 {
                 for member in &self.import_from_stmt.names {
                     if let Some(target) = TYPING_TO_RENAME_PY39.iter().find_map(|(name, target)| {
                         if &member.name == *name {
@@ -468,7 +473,7 @@ impl<'a> ImportReplacer<'a> {
             "typing_extensions" => {
                 // `typing_extensions` to `collections.abc`
                 let mut typing_extensions_to_collections_abc = vec![];
-                if self.version >= PythonVersion::Py312 {
+                if self.version >= PythonVersion::PY312 {
                     typing_extensions_to_collections_abc
                         .extend(TYPING_EXTENSIONS_TO_COLLECTIONS_ABC_312);
                 }
@@ -480,7 +485,7 @@ impl<'a> ImportReplacer<'a> {
 
                 // `typing_extensions` to `warnings`
                 let mut typing_extensions_to_warnings = vec![];
-                if self.version >= PythonVersion::Py313 {
+                if self.version >= PythonVersion::PY313 {
                     typing_extensions_to_warnings.extend(TYPING_EXTENSIONS_TO_WARNINGS_313);
                 }
                 if let Some(operation) =
@@ -491,10 +496,10 @@ impl<'a> ImportReplacer<'a> {
 
                 // `typing_extensions` to `types`
                 let mut typing_extensions_to_types = vec![];
-                if self.version >= PythonVersion::Py312 {
+                if self.version >= PythonVersion::PY312 {
                     typing_extensions_to_types.extend(TYPING_EXTENSIONS_TO_TYPES_312);
                 }
-                if self.version >= PythonVersion::Py313 {
+                if self.version >= PythonVersion::PY313 {
                     typing_extensions_to_types.extend(TYPING_EXTENSIONS_TO_TYPES_313);
                 }
                 if let Some(operation) = self.try_replace(&typing_extensions_to_types, "types") {
@@ -503,25 +508,25 @@ impl<'a> ImportReplacer<'a> {
 
                 // `typing_extensions` to `typing`
                 let mut typing_extensions_to_typing = TYPING_EXTENSIONS_TO_TYPING.to_vec();
-                if self.version >= PythonVersion::Py37 {
+                if self.version >= PythonVersion::PY37 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_37);
                 }
-                if self.version >= PythonVersion::Py38 {
+                if self.version >= PythonVersion::PY38 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_38);
                 }
-                if self.version >= PythonVersion::Py39 {
+                if self.version >= PythonVersion::PY39 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_39);
                 }
-                if self.version >= PythonVersion::Py310 {
+                if self.version >= PythonVersion::PY310 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_310);
                 }
-                if self.version >= PythonVersion::Py311 {
+                if self.version >= PythonVersion::PY311 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_311);
                 }
-                if self.version >= PythonVersion::Py312 {
+                if self.version >= PythonVersion::PY312 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_312);
                 }
-                if self.version >= PythonVersion::Py313 {
+                if self.version >= PythonVersion::PY313 {
                     typing_extensions_to_typing.extend(TYPING_EXTENSIONS_TO_TYPING_313);
                 }
                 if let Some(operation) = self.try_replace(&typing_extensions_to_typing, "typing") {
@@ -530,10 +535,10 @@ impl<'a> ImportReplacer<'a> {
             }
             "mypy_extensions" => {
                 let mut mypy_extensions_to_typing = vec![];
-                if self.version >= PythonVersion::Py37 {
+                if self.version >= PythonVersion::PY37 {
                     mypy_extensions_to_typing.extend(MYPY_EXTENSIONS_TO_TYPING_37);
                 }
-                if self.version >= PythonVersion::Py38 {
+                if self.version >= PythonVersion::PY38 {
                     mypy_extensions_to_typing.extend(MYPY_EXTENSIONS_TO_TYPING_38);
                 }
                 if let Some(operation) = self.try_replace(&mypy_extensions_to_typing, "typing") {
@@ -543,10 +548,10 @@ impl<'a> ImportReplacer<'a> {
             "typing" => {
                 // `typing` to `collections.abc`
                 let mut typing_to_collections_abc = vec![];
-                if self.version >= PythonVersion::Py39 {
+                if self.version >= PythonVersion::PY39 {
                     typing_to_collections_abc.extend(TYPING_TO_COLLECTIONS_ABC_39);
                 }
-                if self.version >= PythonVersion::Py310 {
+                if self.version >= PythonVersion::PY310 {
                     typing_to_collections_abc.extend(TYPING_TO_COLLECTIONS_ABC_310);
                 }
                 if let Some(operation) =
@@ -557,7 +562,7 @@ impl<'a> ImportReplacer<'a> {
 
                 // `typing` to `collections`
                 let mut typing_to_collections = vec![];
-                if self.version >= PythonVersion::Py39 {
+                if self.version >= PythonVersion::PY39 {
                     typing_to_collections.extend(TYPING_TO_COLLECTIONS_39);
                 }
                 if let Some(operation) = self.try_replace(&typing_to_collections, "collections") {
@@ -566,19 +571,19 @@ impl<'a> ImportReplacer<'a> {
 
                 // `typing` to `re`
                 let mut typing_to_re = vec![];
-                if self.version >= PythonVersion::Py39 {
+                if self.version >= PythonVersion::PY39 {
                     typing_to_re.extend(TYPING_TO_RE_39);
                 }
                 if let Some(operation) = self.try_replace(&typing_to_re, "re") {
                     operations.push(operation);
                 }
             }
-            "typing.re" if self.version >= PythonVersion::Py39 => {
+            "typing.re" if self.version >= PythonVersion::PY39 => {
                 if let Some(operation) = self.try_replace(TYPING_RE_TO_RE_39, "re") {
                     operations.push(operation);
                 }
             }
-            "backports.strenum" if self.version >= PythonVersion::Py311 => {
+            "backports.strenum" if self.version >= PythonVersion::PY311 => {
                 if let Some(operation) = self.try_replace(BACKPORTS_STR_ENUM_TO_ENUM_311, "enum") {
                     operations.push(operation);
                 }
@@ -617,7 +622,7 @@ impl<'a> ImportReplacer<'a> {
             let fix = Some(matched);
             Some((operation, fix))
         } else {
-            let indentation = indentation(self.locator, self.import_from_stmt);
+            let indentation = indentation(self.locator.contents(), self.import_from_stmt);
 
             // If we have matched _and_ unmatched names, but the import is not on its own
             // line, we can't add a statement after it. For example, if we have
@@ -694,7 +699,7 @@ impl<'a> ImportReplacer<'a> {
 }
 
 /// UP035
-pub(crate) fn deprecated_import(checker: &mut Checker, import_from_stmt: &StmtImportFrom) {
+pub(crate) fn deprecated_import(checker: &Checker, import_from_stmt: &StmtImportFrom) {
     // Avoid relative and star imports.
     if import_from_stmt.level > 0 {
         return;
@@ -720,11 +725,11 @@ pub(crate) fn deprecated_import(checker: &mut Checker, import_from_stmt: &StmtIm
         checker.locator(),
         checker.stylist(),
         checker.tokens(),
-        checker.settings.target_version,
+        checker.target_version(),
     );
 
     for (operation, fix) in fixer.without_renames() {
-        let mut diagnostic = Diagnostic::new(
+        let mut diagnostic = checker.report_diagnostic(
             DeprecatedImport {
                 deprecation: Deprecation::WithoutRename(operation),
             },
@@ -736,16 +741,14 @@ pub(crate) fn deprecated_import(checker: &mut Checker, import_from_stmt: &StmtIm
                 import_from_stmt.range(),
             )));
         }
-        checker.diagnostics.push(diagnostic);
     }
 
     for operation in fixer.with_renames() {
-        let diagnostic = Diagnostic::new(
+        checker.report_diagnostic(
             DeprecatedImport {
                 deprecation: Deprecation::WithRename(operation),
             },
             import_from_stmt.range(),
         );
-        checker.diagnostics.push(diagnostic);
     }
 }
