@@ -1,8 +1,8 @@
-use ruff_diagnostics::{Diagnostic, Edit, Fix, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, Expr, ExprAttribute, ExprCall};
 use ruff_text_size::Ranged;
 
+use crate::{Edit, Fix, FixAvailability, Violation};
 use crate::{checkers::ast::Checker, importer::ImportRequest};
 
 /// ## What it does
@@ -25,18 +25,24 @@ use crate::{checkers::ast::Checker, importer::ImportRequest};
 /// ## References
 /// - [Python documentation: `Path.cwd`](https://docs.python.org/3/library/pathlib.html#pathlib.Path.cwd)
 
-#[violation]
-pub struct ImplicitCwd;
+#[derive(ViolationMetadata)]
+pub(crate) struct ImplicitCwd;
 
 impl Violation for ImplicitCwd {
+    const FIX_AVAILABILITY: FixAvailability = FixAvailability::Sometimes;
+
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("Prefer `Path.cwd()` over `Path().resolve()` for current-directory lookups")
+        "Prefer `Path.cwd()` over `Path().resolve()` for current-directory lookups".to_string()
+    }
+
+    fn fix_title(&self) -> Option<String> {
+        Some("Replace `Path().resolve()` with `Path.cwd()`".to_string())
     }
 }
 
 /// FURB177
-pub(crate) fn no_implicit_cwd(checker: &mut Checker, call: &ExprCall) {
+pub(crate) fn no_implicit_cwd(checker: &Checker, call: &ExprCall) {
     if !call.arguments.is_empty() {
         return;
     }
@@ -82,7 +88,7 @@ pub(crate) fn no_implicit_cwd(checker: &mut Checker, call: &ExprCall) {
         return;
     }
 
-    let mut diagnostic = Diagnostic::new(ImplicitCwd, call.range());
+    let mut diagnostic = checker.report_diagnostic(ImplicitCwd, call.range());
 
     diagnostic.try_set_fix(|| {
         let (import_edit, binding) = checker.importer().get_or_import_symbol(
@@ -95,8 +101,4 @@ pub(crate) fn no_implicit_cwd(checker: &mut Checker, call: &ExprCall) {
             [import_edit],
         ))
     });
-
-    checker
-        .diagnostics
-        .push(Diagnostic::new(ImplicitCwd, call.range()));
 }
