@@ -1,11 +1,12 @@
 use anyhow::Result;
-use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_diagnostics::Applicability;
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::{self as ast, Expr, Number};
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
 use crate::importer::ImportRequest;
+use crate::{Edit, Fix, FixAvailability, Violation};
 
 /// ## What it does
 /// Checks for `math.log` calls with a redundant base.
@@ -36,13 +37,20 @@ use crate::importer::ImportRequest;
 /// math.log10(4)
 /// ```
 ///
+/// ## Fix safety
+/// This fix is marked unsafe when the argument is a starred expression, as this changes
+/// the call semantics and may raise runtime errors. It is also unsafe if comments are
+/// present within the call, as they will be removed. Additionally, `math.log(x, base)`
+/// and `math.log2(x)` / `math.log10(x)` may differ due to floating-point rounding, so
+/// the fix is also unsafe when making this transformation.
+///
 /// ## References
 /// - [Python documentation: `math.log`](https://docs.python.org/3/library/math.html#math.log)
 /// - [Python documentation: `math.log2`](https://docs.python.org/3/library/math.html#math.log2)
 /// - [Python documentation: `math.log10`](https://docs.python.org/3/library/math.html#math.log10)
 /// - [Python documentation: `math.e`](https://docs.python.org/3/library/math.html#math.e)
-#[violation]
-pub struct RedundantLogBase {
+#[derive(ViolationMetadata)]
+pub(crate) struct RedundantLogBase {
     base: Base,
     arg: String,
 }
@@ -65,7 +73,7 @@ impl Violation for RedundantLogBase {
 }
 
 /// FURB163
-pub(crate) fn redundant_log_base(checker: &mut Checker, call: &ast::ExprCall) {
+pub(crate) fn redundant_log_base(checker: &Checker, call: &ast::ExprCall) {
     if !call.arguments.keywords.is_empty() {
         return;
     }
@@ -77,7 +85,6 @@ pub(crate) fn redundant_log_base(checker: &mut Checker, call: &ast::ExprCall) {
     if !checker
         .semantic()
         .resolve_qualified_name(&call.func)
-        .as_ref()
         .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["math", "log"]))
     {
         return;
@@ -90,7 +97,6 @@ pub(crate) fn redundant_log_base(checker: &mut Checker, call: &ast::ExprCall) {
     } else if checker
         .semantic()
         .resolve_qualified_name(base)
-        .as_ref()
         .is_some_and(|qualified_name| matches!(qualified_name.segments(), ["math", "e"]))
     {
         Base::E
@@ -98,7 +104,7 @@ pub(crate) fn redundant_log_base(checker: &mut Checker, call: &ast::ExprCall) {
         return;
     };
 
-    let mut diagnostic = Diagnostic::new(
+    let mut diagnostic = checker.report_diagnostic(
         RedundantLogBase {
             base,
             arg: checker.locator().slice(arg).into(),
@@ -106,7 +112,6 @@ pub(crate) fn redundant_log_base(checker: &mut Checker, call: &ast::ExprCall) {
         call.range(),
     );
     diagnostic.try_set_fix(|| generate_fix(checker, call, base, arg));
-    checker.diagnostics.push(diagnostic);
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -131,7 +136,7 @@ fn is_number_literal(expr: &Expr, value: i8) -> bool {
         if let Number::Int(number) = &number_literal.value {
             return number.as_i8().is_some_and(|number| number == value);
         } else if let Number::Float(number) = number_literal.value {
-            #[allow(clippy::float_cmp)]
+            #[expect(clippy::float_cmp)]
             return number == f64::from(value);
         }
     }
@@ -144,9 +149,19 @@ fn generate_fix(checker: &Checker, call: &ast::ExprCall, base: Base, arg: &Expr)
         call.start(),
         checker.semantic(),
     )?;
+
     let number = checker.locator().slice(arg);
-    Ok(Fix::safe_edits(
+
+    Ok(Fix::applicable_edits(
         Edit::range_replacement(format!("{binding}({number})"), call.range()),
         [edit],
+        if (matches!(base, Base::Two | Base::Ten))
+            || arg.is_starred_expr()
+            || checker.comment_ranges().intersects(call.range())
+        {
+            Applicability::Unsafe
+        } else {
+            Applicability::Safe
+        },
     ))
 }
