@@ -1,15 +1,15 @@
-use ruff_diagnostics::{Diagnostic, Violation};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::relocate::relocate_expr;
 use ruff_python_ast::visitor::{self, Visitor};
 use ruff_python_ast::{self as ast, Expr, Stmt};
 use ruff_python_codegen::Generator;
 use ruff_text_size::{Ranged, TextRange};
 
+use crate::Violation;
 use crate::checkers::ast::Checker;
 use crate::fix::snippet::SourceCodeSnippet;
 
-use super::super::helpers::{find_file_opens, FileOpen};
+use super::super::helpers::{FileOpen, find_file_opens};
 
 /// ## What it does
 /// Checks for uses of `open` and `write` that can be replaced by `pathlib`
@@ -36,8 +36,8 @@ use super::super::helpers::{find_file_opens, FileOpen};
 /// ## References
 /// - [Python documentation: `Path.write_bytes`](https://docs.python.org/3/library/pathlib.html#pathlib.Path.write_bytes)
 /// - [Python documentation: `Path.write_text`](https://docs.python.org/3/library/pathlib.html#pathlib.Path.write_text)
-#[violation]
-pub struct WriteWholeFile {
+#[derive(ViolationMetadata)]
+pub(crate) struct WriteWholeFile {
     filename: SourceCodeSnippet,
     suggestion: SourceCodeSnippet,
 }
@@ -52,67 +52,41 @@ impl Violation for WriteWholeFile {
 }
 
 /// FURB103
-pub(crate) fn write_whole_file(checker: &mut Checker, with: &ast::StmtWith) {
+pub(crate) fn write_whole_file(checker: &Checker, with: &ast::StmtWith) {
     // `async` check here is more of a precaution.
     if with.is_async {
         return;
     }
 
     // First we go through all the items in the statement and find all `open` operations.
-    let candidates = find_file_opens(with, checker.semantic(), false);
+    let candidates = find_file_opens(with, checker.semantic(), false, checker.target_version());
     if candidates.is_empty() {
         return;
     }
 
     // Then we need to match each `open` operation with exactly one `write` call.
-    let (matches, contents) = {
-        let mut matcher = WriteMatcher::new(candidates);
-        visitor::walk_body(&mut matcher, &with.body);
-        matcher.finish()
-    };
-
-    // All the matched operations should be reported.
-    let diagnostics: Vec<Diagnostic> = matches
-        .iter()
-        .zip(contents)
-        .map(|(open, content)| {
-            Diagnostic::new(
-                WriteWholeFile {
-                    filename: SourceCodeSnippet::from_str(&checker.generator().expr(open.filename)),
-                    suggestion: make_suggestion(open, content, checker.generator()),
-                },
-                open.item.range(),
-            )
-        })
-        .collect();
-    checker.diagnostics.extend(diagnostics);
+    let mut matcher = WriteMatcher::new(checker, candidates);
+    visitor::walk_body(&mut matcher, &with.body);
 }
 
 /// AST visitor that matches `open` operations with the corresponding `write` calls.
-#[derive(Debug)]
-struct WriteMatcher<'a> {
+struct WriteMatcher<'a, 'b> {
+    checker: &'a Checker<'b>,
     candidates: Vec<FileOpen<'a>>,
-    matches: Vec<FileOpen<'a>>,
-    contents: Vec<&'a Expr>,
     loop_counter: u32,
 }
 
-impl<'a> WriteMatcher<'a> {
-    fn new(candidates: Vec<FileOpen<'a>>) -> Self {
+impl<'a, 'b> WriteMatcher<'a, 'b> {
+    fn new(checker: &'a Checker<'b>, candidates: Vec<FileOpen<'a>>) -> Self {
         Self {
+            checker,
             candidates,
-            matches: vec![],
-            contents: vec![],
             loop_counter: 0,
         }
     }
-
-    fn finish(self) -> (Vec<FileOpen<'a>>, Vec<&'a Expr>) {
-        (self.matches, self.contents)
-    }
 }
 
-impl<'a> Visitor<'a> for WriteMatcher<'a> {
+impl<'a> Visitor<'a> for WriteMatcher<'a, '_> {
     fn visit_stmt(&mut self, stmt: &'a Stmt) {
         if matches!(stmt, ast::Stmt::While(_) | ast::Stmt::For(_)) {
             self.loop_counter += 1;
@@ -131,8 +105,16 @@ impl<'a> Visitor<'a> for WriteMatcher<'a> {
                 .position(|open| open.is_ref(write_to))
             {
                 if self.loop_counter == 0 {
-                    self.matches.push(self.candidates.remove(open));
-                    self.contents.push(content);
+                    let open = self.candidates.remove(open);
+                    self.checker.report_diagnostic(
+                        WriteWholeFile {
+                            filename: SourceCodeSnippet::from_str(
+                                &self.checker.generator().expr(open.filename),
+                            ),
+                            suggestion: make_suggestion(&open, content, self.checker.generator()),
+                        },
+                        open.item.range(),
+                    );
                 } else {
                     self.candidates.remove(open);
                 }
@@ -166,6 +148,7 @@ fn make_suggestion(open: &FileOpen<'_>, arg: &Expr, generator: Generator) -> Sou
         id: open.mode.pathlib_method(),
         ctx: ast::ExprContext::Load,
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::dummy(),
     };
     let mut arg = arg.clone();
     relocate_expr(&mut arg, TextRange::default());
@@ -175,8 +158,10 @@ fn make_suggestion(open: &FileOpen<'_>, arg: &Expr, generator: Generator) -> Sou
             args: Box::new([arg]),
             keywords: open.keywords.iter().copied().cloned().collect(),
             range: TextRange::default(),
+            node_index: ruff_python_ast::AtomicNodeIndex::dummy(),
         },
         range: TextRange::default(),
+        node_index: ruff_python_ast::AtomicNodeIndex::dummy(),
     };
     SourceCodeSnippet::from_str(&generator.expr(&call.into()))
 }
