@@ -1,10 +1,12 @@
-use crate::server::{client::Notifier, Result};
-use crate::session::DocumentSnapshot;
+use crate::server::Result;
+use crate::session::{Client, DocumentSnapshot};
+use anyhow::Context;
 use lsp_types::{self as types, request as req};
 use regex::Regex;
-use ruff_diagnostics::FixAvailability;
+use ruff_linter::FixAvailability;
 use ruff_linter::registry::{Linter, Rule, RuleNamespace};
 use ruff_source_file::OneIndexed;
+use std::fmt::Write;
 
 pub(crate) struct Hover;
 
@@ -18,7 +20,7 @@ impl super::BackgroundDocumentRequestHandler for Hover {
     }
     fn run_with_snapshot(
         snapshot: DocumentSnapshot,
-        _notifier: Notifier,
+        _client: &Client,
         params: types::HoverParams,
     ) -> Result<Option<types::Hover>> {
         Ok(hover(&snapshot, &params.text_document_position_params))
@@ -33,7 +35,8 @@ pub(crate) fn hover(
     let document = snapshot
         .query()
         .as_single_document()
-        .expect("hover should only be called on text documents or notebook cells");
+        .context("Failed to get text document for the hover request")
+        .unwrap();
     let line_number: usize = position
         .position
         .line
@@ -82,12 +85,16 @@ pub(crate) fn hover(
 
 fn format_rule_text(rule: Rule) -> String {
     let mut output = String::new();
-    output.push_str(&format!("# {} ({})", rule.as_ref(), rule.noqa_code()));
+    let _ = write!(&mut output, "# {} ({})", rule.name(), rule.noqa_code());
     output.push('\n');
     output.push('\n');
 
     let (linter, _) = Linter::parse_code(&rule.noqa_code().to_string()).unwrap();
-    output.push_str(&format!("Derived from the **{}** linter.", linter.name()));
+    let _ = write!(
+        &mut output,
+        "Derived from the **{}** linter.",
+        linter.name()
+    );
     output.push('\n');
     output.push('\n');
 
@@ -101,7 +108,7 @@ fn format_rule_text(rule: Rule) -> String {
         output.push('\n');
     }
 
-    if rule.is_preview() || rule.is_nursery() {
+    if rule.is_preview() {
         output.push_str(r"This rule is in preview and is not stable.");
         output.push('\n');
         output.push('\n');
